@@ -2,7 +2,13 @@
 set -Eeuo pipefail
 # Check that issues in CHANGELOG are up-to-to-date
 
-version=$(head -1 <CHANGELOG.md| cut -d" " -f1 | cut -d'[' -f2)
+first_line=$(head -1 <CHANGELOG.md)
+version=$(echo "$first_line" | cut -d" " -f1 | cut -d'[' -f2)
+if [[ -z "$version" ]]; then
+    echo "Version can not be extracted from the first line <$first_line> of the CHANGELOG"
+    exit 1
+fi
+
 echo "version=$version"
 changelog=$(bin/release/changelog_extract.py "$version" | grep '^\* ')
 issues=$(bin/release/get_release_note.py "$version" | grep '^\* ')
