@@ -1,56 +1,23 @@
 #!/usr/bin/env python3
+from dataclasses import dataclass
 from typing import *
 
 
+@dataclass
 class Option:
-    def __init__(
-        self,
-        name,
-        long,
-        short,
-        value,
-        value_default,
-        value_parser,
-        help,
-        conflict,
-        append,
-        cli_only,
-        deprecated,
-        experimental,
-        description,
-    ):
-        self.name = name
-        self.long = long
-        self.short = short
-        self.value = value
-        self.value_default = value_default
-        self.value_parser = value_parser
-        self.help = help
-        self.conflict = conflict
-        self.append = append
-        self.cli_only = cli_only
-        self.deprecated = deprecated
-        self.experimental = experimental
-        self.description = description
-
-    def __eq__(self, other):
-        if not isinstance(other, Option):
-            return False
-        return (
-            self.name == other.name
-            and self.long == other.long
-            and self.short == other.short
-            and self.value == other.value
-            and self.value_default == other.value_default
-            and self.value_parser == other.value_parser
-            and self.help == other.help
-            and self.conflict == other.conflict
-            and self.append == other.append
-            and self.deprecated == other.deprecated
-            and self.experimental == other.experimental
-            and self.description == other.description
-            and self.cli_only == other.cli_only
-        )
+    name: str
+    long: str
+    description: str
+    short: Optional[str] = None
+    value: Optional[str] = None
+    value_default: Optional[str] = None
+    value_parser: Optional[str] = None
+    help: Optional[str] = None
+    conflict: Optional[str] = None
+    append: bool = False
+    cli_only: bool = False
+    deprecated: bool = False
+    experimental: bool = False
 
     def __str__(self):
         s = "name: " + self.name
@@ -79,25 +46,6 @@ class Option:
         s += "\n" + self.description
         return s
 
-    def __hash__(self):
-        return hash(
-            (
-                self.name,
-                self.long,
-                self.short,
-                self.value,
-                self.value_default,
-                self.value_parser,
-                self.help,
-                self.conflict,
-                self.append,
-                self.cli_only,
-                self.deprecated,
-                self.experimental,
-                self.description,
-            )
-        )
-
     @staticmethod
     def parse(s) -> "Option":
         name = None
@@ -136,6 +84,8 @@ class Option:
                     value_parser = v
                 elif key == "help":
                     help = v
+                    if help.endswith("."):
+                        raise Exception(f"{name}: help should not end with period")
                 elif key == "conflict":
                     conflict = [a.strip() for a in v.split(" ")]
                 elif key == "multi":
@@ -147,7 +97,9 @@ class Option:
                     elif v == "false":
                         cli_only = False
                     else:
-                        raise Exception("Expected true or false for cli attribute")
+                        raise Exception(
+                            f"{name}: Expected true or false for cli attribute"
+                        )
                 elif key == "deprecated":
                     if v == "true":
                         deprecated = True
@@ -155,7 +107,7 @@ class Option:
                         deprecated = False
                     else:
                         raise Exception(
-                            "Expected true or false for deprecated attribute"
+                            f"{name}: Expected true or false for deprecated attribute"
                         )
                 elif key == "experimental":
                     if v == "true":
@@ -164,31 +116,31 @@ class Option:
                         experimental = False
                     else:
                         raise Exception(
-                            "Expected true or false for experimental attribute"
+                            f"{name}: Expected true or false for experimental attribute"
                         )
                 else:
-                    raise Exception("Invalid attribute " + key)
+                    raise Exception(f"{name}: Invalid attribute " + key)
 
         if name is None:
             raise Exception("missing name attribute")
 
         if long is None:
-            raise Exception("missing long attribute")
+            raise Exception(f"{name}: missing long attribute")
 
         return Option(
-            name,
-            long,
-            short,
-            value,
-            value_default,
-            value_parser,
-            help,
-            conflict,
-            append,
-            cli_only,
-            deprecated,
-            experimental,
-            description.strip(),
+            name=name,
+            long=long,
+            short=short,
+            value=value,
+            value_default=value_default,
+            value_parser=value_parser,
+            help=help,
+            conflict=conflict,
+            append=append,
+            cli_only=cli_only,
+            deprecated=deprecated,
+            experimental=experimental,
+            description=description.strip(),
         )
 
     @staticmethod
