@@ -26,7 +26,7 @@ def bold_red(text: str) -> str:
     return f"\x1b[1;31m{text}\x1b[0m"
 
 
-def check_archive(version: str):
+def check_archive(hurl_version: str, package_version: str):
     print(bold_blue("Checking archives:"))
     path = Path("contrib/npm/hurl/platform.json")
     platforms = json.loads(path.read_text())
@@ -35,7 +35,7 @@ def check_archive(version: str):
         target = platform["rust_target"]
         extension = platform["archive_extension"]
         expected_checksum = platform["checksum"]
-        url = f"https://github.com/Orange-OpenSource/hurl/releases/download/{version}/hurl-{version}-{target}{extension}"
+        url = f"https://github.com/Orange-OpenSource/hurl/releases/download/{hurl_version}/hurl-{hurl_version}-{target}{extension}"
         print(f"  Downloading: {bold(url)}")
         with request.urlopen(url) as response:
             if response.status != 200:
@@ -60,37 +60,51 @@ def check_archive(version: str):
         print()
 
 
-def check_version(version: str):
+def check_version(hurl_version: str, package_version: str):
     print(bold_blue("Checking version:"))
     path = Path("contrib/npm/hurl/package.json")
     package = json.loads(path.read_text())
-    expected_version = version
-    actual_version = package["version"]
-    if actual_version != expected_version:
+    expected_hurl_version = hurl_version
+    actual_hurl_version = package["hurlBinaryVersion"]
+    expected_package_version = package_version
+    actual_package_version = package["version"]
+
+    if actual_hurl_version != expected_hurl_version:
         print(
             bold_red(
-                f"  Version KO actual={actual_version} expected={expected_version}, please update "
+                f"  Hurl version KO actual={actual_hurl_version} expected={expected_hurl_version}, please update "
+                f"hurlBinaryVersion in contrib/npm/hurl/package.json"
+            )
+        )
+        sys.exit(1)
+    else:
+        print(bold_green("  Hurl version OK"))
+
+    if actual_package_version != expected_package_version:
+        print(
+            bold_red(
+                f"  Package version KO actual={actual_package_version} expected={expected_package_version}, please update "
                 f"version in contrib/npm/hurl/package.json"
             )
         )
         sys.exit(1)
     else:
-        print(bold_green("  Version OK"))
+        print(bold_green("  Package version OK"))
 
 
-def check_manual(version: str):
+def check_manual(hurl_version: str, package_version: str):
     print(bold_blue("Checking manual:"))
     print()
     pass
 
 
-def main(version: str):
-    check_version(version)
-    check_manual(version)
-    check_archive(version)
+def main(hurl_version: str, package_version):
+    check_version(hurl_version, package_version)
+    check_manual(hurl_version, package_version)
+    check_archive(hurl_version, package_version)
 
     print(bold("Everything looks OK!"))
 
 
 if __name__ == "__main__":
-    main(sys.argv[1])
+    main(sys.argv[1], sys.argv[2])
