@@ -20,14 +20,15 @@ use std::fs::File;
 use std::io::{BufRead, BufReader, IsTerminal};
 use std::path::{Path, PathBuf};
 use std::time::Duration;
-use std::{env, io};
+use std::{env, fs, io};
 
 use clap::ArgMatches;
-use hurl::runner::{Input, Value};
-use hurl_core::ast::Retry;
+use hurl::runner::Value;
+use hurl_core::input::Input;
+use hurl_core::typing::{Count, DurationUnit};
 
-use super::variables::{parse as parse_variable, parse_value};
-use super::CliOptionsError;
+use crate::cli::options::variables::{parse as parse_variable, parse_value};
+use crate::cli::options::{duration, CliOptionsError};
 use crate::cli::options::{ErrorFormat, HttpVersion, IpResolve, Output};
 use crate::cli::OutputType;
 
@@ -40,7 +41,7 @@ pub fn cacert_file(arg_matches: &ArgMatches) -> Result<Option<String>, CliOption
                 Ok(Some(filename))
             } else {
                 Err(CliOptionsError::Error(format!(
-                    "input file {} does not exist",
+                    "Input file {} does not exist",
                     path.display()
                 )))
             }
@@ -100,9 +101,9 @@ pub fn compressed(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "compressed")
 }
 
-pub fn connect_timeout(arg_matches: &ArgMatches) -> Duration {
-    let value = get::<u64>(arg_matches, "connect_timeout").unwrap();
-    Duration::from_secs(value)
+pub fn connect_timeout(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
+    let s = get::<String>(arg_matches, "connect_timeout").unwrap_or_default();
+    get_duration(&s, DurationUnit::Second)
 }
 
 pub fn connects_to(arg_matches: &ArgMatches) -> Vec<String> {
@@ -125,9 +126,9 @@ pub fn cookie_output_file(arg_matches: &ArgMatches) -> Option<PathBuf> {
     get::<String>(arg_matches, "cookies_output_file").map(PathBuf::from)
 }
 
-pub fn delay(arg_matches: &ArgMatches) -> Duration {
-    let millis = get::<u64>(arg_matches, "delay").unwrap();
-    Duration::from_millis(millis)
+pub fn delay(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
+    let s = get::<String>(arg_matches, "delay").unwrap_or_default();
+    get_duration(&s, DurationUnit::MilliSecond)
 }
 
 pub fn error_format(arg_matches: &ArgMatches) -> ErrorFormat {
@@ -158,7 +159,7 @@ pub fn html_dir(arg_matches: &ArgMatches) -> Result<Option<PathBuf>, CliOptionsE
     if let Some(dir) = get::<String>(arg_matches, "report_html") {
         let path = Path::new(&dir);
         if !path.exists() {
-            match std::fs::create_dir_all(path) {
+            match fs::create_dir_all(path) {
                 Err(_) => Err(CliOptionsError::Error(format!(
                     "HTML dir {} can not be created",
                     path.display()
@@ -200,16 +201,29 @@ pub fn include(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "include")
 }
 
+/// Returns true if we have at least one input files.
+/// The input file can be a file, the standard input, or a glob (even a glob returns empty results).
+pub fn has_input_files(arg_matches: &ArgMatches) -> bool {
+    get_strings(arg_matches, "input_files").is_some()
+        || get_strings(arg_matches, "glob").is_some()
+        || !io::stdin().is_terminal()
+}
+
 /// Returns the input files from the positional arguments and the glob options
 pub fn input_files(arg_matches: &ArgMatches) -> Result<Vec<Input>, CliOptionsError> {
     let mut files = vec![];
     if let Some(filenames) = get_strings(arg_matches, "input_files") {
         for filename in &filenames {
-            let file = Input::new(filename);
-            if !file.exists() {
-                return Err(CliOptionsError::InvalidInputFile(PathBuf::from(filename)));
+            let filename = Path::new(filename);
+            if !filename.exists() {
+                return Err(CliOptionsError::InvalidInputFile(filename.to_path_buf()));
+            }
+            if filename.is_file() {
+                let file = Input::from(filename);
+                files.push(file);
+            } else if filename.is_dir() {
+                walks_hurl_files(filename, &mut files)?;
             }
-            files.push(file);
         }
     }
     for filename in glob_files(arg_matches)? {
@@ -221,6 +235,25 @@ pub fn input_files(arg_matches: &ArgMatches) -> Result<Vec<Input>, CliOptionsErr
     Ok(files)
 }
 
+/// Walks recursively a directory from `dir` and push Hurl files to `files`.
+fn walks_hurl_files(dir: &Path, files: &mut Vec<Input>) -> Result<(), CliOptionsError> {
+    let Ok(entries) = fs::read_dir(dir) else {
+        return Err(CliOptionsError::InvalidInputFile(dir.to_path_buf()));
+    };
+    for entry in entries {
+        let Ok(entry) = entry else {
+            return Err(CliOptionsError::InvalidInputFile(dir.to_path_buf()));
+        };
+        let path = entry.path();
+        if path.is_dir() {
+            walks_hurl_files(&path, files)?;
+        } else if entry.path().extension() == Some("hurl".as_ref()) {
+            files.push(Input::from(entry.path()));
+        }
+    }
+    Ok(())
+}
+
 pub fn insecure(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "insecure")
 }
@@ -243,10 +276,10 @@ pub fn junit_file(arg_matches: &ArgMatches) -> Option<PathBuf> {
     get::<String>(arg_matches, "report_junit").map(PathBuf::from)
 }
 
-pub fn max_redirect(arg_matches: &ArgMatches) -> Option<usize> {
+pub fn max_redirect(arg_matches: &ArgMatches) -> Count {
     match get::<i32>(arg_matches, "max_redirects").unwrap() {
-        m if m == -1 => None,
-        m => Some(m as usize),
+        -1 => Count::Infinite,
+        m => Count::Finite(m as usize),
     }
 }
 
@@ -254,6 +287,30 @@ pub fn jobs(arg_matches: &ArgMatches) -> Option<usize> {
     get::<u32>(arg_matches, "jobs").map(|m| m as usize)
 }
 
+pub fn json_report_dir(arg_matches: &ArgMatches) -> Result<Option<PathBuf>, CliOptionsError> {
+    if let Some(dir) = get::<String>(arg_matches, "report_json") {
+        let path = Path::new(&dir);
+        if !path.exists() {
+            match fs::create_dir_all(path) {
+                Err(_) => Err(CliOptionsError::Error(format!(
+                    "JSON dir {} can not be created",
+                    path.display()
+                ))),
+                Ok(_) => Ok(Some(path.to_path_buf())),
+            }
+        } else if path.is_dir() {
+            Ok(Some(path.to_path_buf()))
+        } else {
+            return Err(CliOptionsError::Error(format!(
+                "{} is not a valid directory",
+                path.display()
+            )));
+        }
+    } else {
+        Ok(None)
+    }
+}
+
 pub fn netrc(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "netrc")
 }
@@ -295,7 +352,7 @@ pub fn output_type(arg_matches: &ArgMatches) -> OutputType {
 }
 
 pub fn parallel(arg_matches: &ArgMatches) -> bool {
-    has_flag(arg_matches, "parallel")
+    has_flag(arg_matches, "parallel") || has_flag(arg_matches, "test")
 }
 
 pub fn path_as_is(arg_matches: &ArgMatches) -> bool {
@@ -310,21 +367,29 @@ pub fn proxy(arg_matches: &ArgMatches) -> Option<String> {
     get::<String>(arg_matches, "proxy")
 }
 
+pub fn repeat(arg_matches: &ArgMatches) -> Option<Count> {
+    match get::<i32>(arg_matches, "repeat") {
+        Some(-1) => Some(Count::Infinite),
+        Some(n) => Some(Count::Finite(n as usize)),
+        None => None,
+    }
+}
+
 pub fn resolves(arg_matches: &ArgMatches) -> Vec<String> {
     get_strings(arg_matches, "resolve").unwrap_or_default()
 }
 
-pub fn retry(arg_matches: &ArgMatches) -> Retry {
-    match get::<i32>(arg_matches, "retry").unwrap() {
-        -1 => Retry::Infinite,
-        0 => Retry::None,
-        r => Retry::Finite(r as usize),
+pub fn retry(arg_matches: &ArgMatches) -> Option<Count> {
+    match get::<i32>(arg_matches, "retry") {
+        Some(-1) => Some(Count::Infinite),
+        Some(r) => Some(Count::Finite(r as usize)),
+        None => None,
     }
 }
 
-pub fn retry_interval(arg_matches: &ArgMatches) -> Duration {
-    let value = get::<u64>(arg_matches, "retry_interval").unwrap();
-    Duration::from_millis(value)
+pub fn retry_interval(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
+    let s = get::<String>(arg_matches, "retry_interval").unwrap_or_default();
+    get_duration(&s, DurationUnit::MilliSecond)
 }
 
 pub fn ssl_no_revoke(arg_matches: &ArgMatches) -> bool {
@@ -339,9 +404,9 @@ pub fn test(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "test")
 }
 
-pub fn timeout(arg_matches: &ArgMatches) -> Duration {
-    let value = get::<u64>(arg_matches, "max_time").unwrap();
-    Duration::from_secs(value)
+pub fn timeout(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
+    let s = get::<String>(arg_matches, "max_time").unwrap_or_default();
+    get_duration(&s, DurationUnit::Second)
 }
 
 pub fn to_entry(arg_matches: &ArgMatches) -> Option<usize> {
@@ -480,6 +545,18 @@ pub fn get_strings(matches: &ArgMatches, name: &str) -> Option<Vec<String>> {
         .map(|v| v.map(|x| x.to_string()).collect())
 }
 
+/// Get duration from input string `s` and `default_unit`
+fn get_duration(s: &str, default_unit: DurationUnit) -> Result<Duration, CliOptionsError> {
+    let duration = duration::parse(s).map_err(CliOptionsError::Error)?;
+    let unit = duration.unit.unwrap_or(default_unit);
+    let millis = match unit {
+        DurationUnit::MilliSecond => duration.value,
+        DurationUnit::Second => duration.value * 1000,
+        DurationUnit::Minute => duration.value * 1000 * 60,
+    };
+    Ok(Duration::from_millis(millis))
+}
+
 /// Whether or not this running in a Continuous Integration environment.
 /// Code borrowed from <https://github.com/rust-lang/cargo/blob/master/crates/cargo-util/src/lib.rs>
 fn is_ci() -> bool {
