@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2023 Orange
+ * Copyright (C) 2024 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,19 +18,25 @@
 use core::fmt;
 
 use crate::http::core::*;
-use crate::http::{header, Header};
+use crate::http::header::HeaderVec;
 
+/// Represents the HTTP request asked to be executed by our user (different from the runtime
+/// executed HTTP request [`crate::http::Request`].
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct RequestSpec {
     pub method: Method,
     pub url: String,
-    pub headers: Vec<Header>,
+    pub headers: HeaderVec,
     pub querystring: Vec<Param>,
     pub form: Vec<Param>,
     pub multipart: Vec<MultipartParam>,
     pub cookies: Vec<RequestCookie>,
     pub body: Body,
-    pub content_type: Option<String>,
+    /// This is the implicit content type of the request: this content type is implicitly set when
+    /// the request use a "typed" body: form, JSON, multipart, multiline string with hint. This
+    /// implicit content type can be different from the user provided one through the `headers`
+    /// field.
+    pub implicit_content_type: Option<String>,
 }
 
 impl Default for RequestSpec {
@@ -38,13 +44,13 @@ impl Default for RequestSpec {
         RequestSpec {
             method: Method("GET".to_string()),
             url: String::new(),
-            headers: vec![],
+            headers: HeaderVec::new(),
             querystring: vec![],
             form: vec![],
             multipart: vec![],
             cookies: vec![],
             body: Body::Binary(vec![]),
-            content_type: None,
+            implicit_content_type: None,
         }
     }
 }
@@ -83,13 +89,6 @@ impl Body {
     }
 }
 
-impl RequestSpec {
-    /// Returns all header values.
-    pub fn get_header_values(&self, name: &str) -> Vec<String> {
-        header::get_values(&self.headers, name)
-    }
-}
-
 impl fmt::Display for Method {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{}", self.0)
