@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2023 Orange
+ * Copyright (C) 2024 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,9 +16,10 @@
  *
  */
 use std::collections::HashMap;
+use std::path::Path;
 
 use crate::http::core::*;
-use crate::http::{RequestSpec, *};
+use crate::http::*;
 use crate::util::path::ContextDir;
 
 impl RequestSpec {
@@ -35,32 +36,28 @@ impl RequestSpec {
             arguments.append(&mut header.curl_args());
         }
 
-        let has_explicit_content_type = self
-            .headers
-            .iter()
-            .map(|h| &h.name)
-            .any(|n| n.as_str() == "Content-Type");
+        let has_explicit_content_type = self.headers.contains_key(CONTENT_TYPE);
         if !has_explicit_content_type {
-            if let Some(content_type) = &self.content_type {
-                if content_type.as_str() != "application/x-www-form-urlencoded"
-                    && content_type.as_str() != "multipart/form-data"
+            if let Some(content_type) = &self.implicit_content_type {
+                if content_type != "application/x-www-form-urlencoded"
+                    && content_type != "multipart/form-data"
                 {
                     arguments.push("--header".to_string());
-                    arguments.push(format!("'Content-Type: {content_type}'"));
+                    arguments.push(format!("'{}: {content_type}'", CONTENT_TYPE));
                 }
             } else if !self.body.bytes().is_empty() {
                 match self.body {
                     Body::Text(_) => {
                         arguments.push("--header".to_string());
-                        arguments.push("'Content-Type:'".to_string())
+                        arguments.push(format!("'{}:'", CONTENT_TYPE));
                     }
                     Body::Binary(_) => {
                         arguments.push("--header".to_string());
-                        arguments.push("'Content-Type: application/octet-stream'".to_string())
+                        arguments.push(format!("'{}: application/octet-stream'", CONTENT_TYPE));
                     }
                     Body::File(_, _) => {
                         arguments.push("--header".to_string());
-                        arguments.push("'Content-Type:'".to_string())
+                        arguments.push(format!("'{}:'", CONTENT_TYPE));
                     }
                 }
             }
@@ -128,8 +125,8 @@ fn encode_byte(b: u8) -> String {
     format!("\\x{b:02x}")
 }
 
-fn encode_bytes(b: Vec<u8>) -> String {
-    b.iter().map(|b| encode_byte(*b)).collect()
+fn encode_bytes(bytes: &[u8]) -> String {
+    bytes.iter().map(|b| encode_byte(*b)).collect()
 }
 
 impl Method {
@@ -157,11 +154,11 @@ impl Method {
 
 impl Header {
     pub fn curl_args(&self) -> Vec<String> {
-        let name = self.name.clone();
-        let value = self.value.clone();
+        let name = &self.name;
+        let value = &self.value;
         vec![
             "--header".to_string(),
-            encode_shell_string(format!("{name}: {value}").as_str()),
+            encode_shell_string(&format!("{name}: {value}")),
         ]
     }
 }
@@ -190,8 +187,8 @@ impl MultipartParam {
                 content_type,
                 ..
             }) => {
-                let path = context_dir.get_path(filename);
-                let value = format!("@{};type={}", path.to_str().unwrap(), content_type);
+                let path = context_dir.resolved_path(Path::new(filename));
+                let value = format!("@{};type={}", path.to_string_lossy(), content_type);
                 format!("{name}={value}")
             }
         }
@@ -200,12 +197,12 @@ impl MultipartParam {
 
 impl Body {
     pub fn curl_arg(&self, context_dir: &ContextDir) -> String {
-        match self.clone() {
-            Body::Text(s) => encode_shell_string(&s),
+        match self {
+            Body::Text(s) => encode_shell_string(s),
             Body::Binary(bytes) => format!("$'{}'", encode_bytes(bytes)),
             Body::File(_, filename) => {
-                let path = context_dir.get_path(&filename);
-                format!("'@{}'", path.to_str().unwrap())
+                let path = context_dir.resolved_path(Path::new(filename));
+                format!("'@{}'", path.to_string_lossy())
             }
         }
     }
@@ -260,13 +257,16 @@ pub mod tests {
     use super::*;
 
     fn form_http_request() -> RequestSpec {
+        let mut headers = HeaderVec::new();
+        headers.push(Header::new(
+            "Content-Type",
+            "application/x-www-form-urlencoded",
+        ));
+
         RequestSpec {
             method: Method("POST".to_string()),
             url: "http://localhost/form-params".to_string(),
-            headers: vec![Header::new(
-                "Content-Type",
-                "application/x-www-form-urlencoded",
-            )],
+            headers,
             form: vec![
                 Param {
                     name: String::from("param1"),
@@ -277,7 +277,20 @@ pub mod tests {
                     value: String::from("a b"),
                 },
             ],
-            content_type: Some("multipart/form-data".to_string()),
+            implicit_content_type: Some("multipart/form-data".to_string()),
+            ..Default::default()
+        }
+    }
+
+    fn json_request() -> RequestSpec {
+        let mut headers = HeaderVec::new();
+        headers.push(Header::new("content-type", "application/vnd.api+json"));
+        RequestSpec {
+            method: Method("POST".to_string()),
+            url: "http://localhost/json".to_string(),
+            headers,
+            body: Body::Text("{\"foo\":\"bar\"}".to_string()),
+            implicit_content_type: Some("application/json".to_string()),
             ..Default::default()
         }
     }
@@ -398,6 +411,16 @@ pub mod tests {
                 "'http://localhost/form-params'".to_string(),
             ]
         );
+        assert_eq!(
+            json_request().curl_args(context_dir),
+            vec![
+                "--header".to_string(),
+                "'content-type: application/vnd.api+json'".to_string(),
+                "--data".to_string(),
+                "'{\"foo\":\"bar\"}'".to_string(),
+                "'http://localhost/json'".to_string(),
+            ]
+        );
     }
 
     #[test]
