@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2023 Orange
+ * Copyright (C) 2024 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -137,7 +137,7 @@ impl Selector {
                 let mut values = vec![];
                 for index in indexes {
                     if let Some(value) = root.get(index) {
-                        values.push(value.clone())
+                        values.push(value.clone());
                     }
                 }
                 Some(JsonpathResult::Collection(values))
