@@ -15,9 +15,10 @@
  * limitations under the License.
  *
  */
+use hurl_core::error::{DisplaySourceError, OutputFormat};
+
 use crate::report::html::Testcase;
-use crate::runner::Error;
-use crate::util::logger;
+use crate::runner::RunnerError;
 
 #[derive(Copy, Clone, Eq, PartialEq)]
 pub enum Tab {
@@ -75,10 +76,20 @@ fn get_status_html(success: bool) -> &'static str {
 }
 
 /// Returns an HTML `<pre>` tag representing this `error`.
-fn error_to_html(error: &Error, content: &str, filename: &str, source_filename: &str) -> String {
+fn error_to_html(
+    error: &RunnerError,
+    content: &str,
+    filename: &str,
+    source_filename: &str,
+) -> String {
     let line = error.source_info.start.line;
     let column = error.source_info.start.column;
-    let message = logger::error_string(filename, content, error, Some(error.source_info), false);
+    let message = error.to_string(
+        filename,
+        content,
+        Some(error.source_info),
+        OutputFormat::Terminal(false),
+    );
     let message = html_escape(&message);
     // We override the first part of the error string to add an anchor to
     // the error context.
@@ -96,15 +107,17 @@ fn html_escape(text: &str) -> String {
 
 #[cfg(test)]
 mod tests {
+    use hurl_core::ast::SourceInfo;
+    use hurl_core::reader::Pos;
+
     use crate::report::html::nav::error_to_html;
-    use crate::runner::{Error, RunnerError};
-    use hurl_core::ast::{Pos, SourceInfo};
+    use crate::runner::{RunnerError, RunnerErrorKind};
 
     #[test]
     fn test_error_html() {
-        let error = Error::new(
+        let error = RunnerError::new(
             SourceInfo::new(Pos::new(4, 1), Pos::new(4, 9)),
-            RunnerError::AssertFailure {
+            RunnerErrorKind::AssertFailure {
                 actual: "<script>alert('Hi')</script>".to_string(),
                 expected: "Hello world".to_string(),
                 type_mismatch: false,
