@@ -15,8 +15,10 @@
  * limitations under the License.
  *
  */
-use crate::report::Error;
-use crate::runner::{HurlResult, Input};
+use hurl_core::input::Input;
+
+use crate::report::ReportError;
+use crate::runner::HurlResult;
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Testcase {
@@ -38,7 +40,7 @@ impl Testcase {
     /// Creates an Tap &lt;testcase&gt; from a TAP line
     /// ok 1 - this is the first test
     /// nok 2 - this is the second test
-    pub(crate) fn parse(line: &str) -> Result<Testcase, Error> {
+    pub(crate) fn parse(line: &str) -> Result<Testcase, ReportError> {
         let mut line = line;
         let success = if line.starts_with("ok") {
             line = &line[2..];
@@ -47,22 +49,22 @@ impl Testcase {
             line = &line[6..];
             false
         } else {
-            return Err(Error {
-                message: format!("Invalid TAP line <{line}> - must start with ok or nok"),
-            });
+            return Err(ReportError::from_string(&format!(
+                "Invalid TAP line <{line}> - must start with ok or nok"
+            )));
         };
 
         let description = match line.find('-') {
             None => {
-                return Err(Error {
-                    message: format!("Invalid TAP line <{line}> - missing '-' separator"),
-                })
+                return Err(ReportError::from_string(&format!(
+                    "Invalid TAP line <{line}> - missing '-' separator"
+                )));
             }
             Some(index) => {
                 if line.split_at(index).0.trim().parse::<usize>().is_err() {
-                    return Err(Error {
-                        message: format!("Invalid TAP line <{line}> - missing test number"),
-                    });
+                    return Err(ReportError::from_string(&format!(
+                        "Invalid TAP line <{line}> - missing test number"
+                    )));
                 }
                 line.split_at(index).1[1..].trim().to_string()
             }
@@ -81,7 +83,7 @@ mod tests {
     #[test]
     fn parse_tap_test_line() {
         assert_eq!(
-            Testcase::parse("toto").err().unwrap().message,
+            Testcase::parse("toto").err().unwrap().to_string(),
             "Invalid TAP line <toto> - must start with ok or nok".to_string()
         );
 
