@@ -20,7 +20,8 @@ use std::collections::HashMap;
 use hurl_core::ast::*;
 
 use crate::http;
-use crate::runner::error::{Error, RunnerError};
+use crate::runner::cache::BodyCache;
+use crate::runner::error::{RunnerError, RunnerErrorKind};
 use crate::runner::filter::eval_filters;
 use crate::runner::query::eval_query;
 use crate::runner::result::CaptureResult;
@@ -28,29 +29,37 @@ use crate::runner::template::eval_template;
 use crate::runner::Value;
 
 /// Evaluates a `capture` with `variables` map and `http_response`, returns a
-/// [`CaptureResult`] on success or an [`Error`] .
+/// [`CaptureResult`] on success or an [`RunnerError`].
+///
+/// The `cache` is used to store XML / JSON structured response data and avoid redundant parsing
+/// operation on the response.
 pub fn eval_capture(
     capture: &Capture,
     variables: &HashMap<String, Value>,
     http_response: &http::Response,
-) -> Result<CaptureResult, Error> {
+    cache: &mut BodyCache,
+) -> Result<CaptureResult, RunnerError> {
     let name = eval_template(&capture.name, variables)?;
-    let value = eval_query(&capture.query, variables, http_response)?;
+    let value = eval_query(&capture.query, variables, http_response, cache)?;
     let value = match value {
         None => {
-            return Err(Error::new(
+            return Err(RunnerError::new(
                 capture.query.source_info,
-                RunnerError::NoQueryResult,
+                RunnerErrorKind::NoQueryResult,
                 false,
             ));
         }
         Some(value) => {
-            let filters = capture.filters.iter().map(|(_, f)| f.clone()).collect();
+            let filters = capture
+                .filters
+                .iter()
+                .map(|(_, f)| f.clone())
+                .collect::<Vec<_>>();
             match eval_filters(&filters, &value, variables, false)? {
                 None => {
-                    return Err(Error::new(
+                    return Err(RunnerError::new(
                         capture.query.source_info,
-                        RunnerError::NoQueryResult,
+                        RunnerErrorKind::NoQueryResult,
                         false,
                     ));
                 }
@@ -67,7 +76,8 @@ pub fn eval_capture(
 
 #[cfg(test)]
 pub mod tests {
-    use hurl_core::ast::{Pos, SourceInfo};
+    use hurl_core::ast::SourceInfo;
+    use hurl_core::reader::Pos;
 
     use self::super::super::query;
     use super::*;
@@ -138,6 +148,7 @@ pub mod tests {
     #[test]
     fn test_invalid_xpath() {
         let variables = HashMap::new();
+        let mut cache = BodyCache::new();
         let whitespace = Whitespace {
             value: String::new(),
             source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
@@ -165,11 +176,16 @@ pub mod tests {
             },
         };
 
-        let error = eval_capture(&capture, &variables, &http::xml_three_users_http_response())
-            .err()
-            .unwrap();
+        let error = eval_capture(
+            &capture,
+            &variables,
+            &http::xml_three_users_http_response(),
+            &mut cache,
+        )
+        .err()
+        .unwrap();
         assert_eq!(error.source_info.start, Pos { line: 1, column: 7 });
-        assert_eq!(error.inner, RunnerError::QueryInvalidXpathEval);
+        assert_eq!(error.kind, RunnerErrorKind::QueryInvalidXpathEval);
     }
 
     #[test]
@@ -220,11 +236,14 @@ pub mod tests {
     #[test]
     fn test_capture() {
         let variables = HashMap::new();
+        let mut cache = BodyCache::new();
+
         assert_eq!(
             eval_capture(
                 &user_count_capture(),
                 &variables,
                 &http::xml_three_users_http_response(),
+                &mut cache,
             )
             .unwrap(),
             CaptureResult {
@@ -234,7 +253,13 @@ pub mod tests {
         );
 
         assert_eq!(
-            eval_capture(&duration_capture(), &variables, &http::json_http_response()).unwrap(),
+            eval_capture(
+                &duration_capture(),
+                &variables,
+                &http::json_http_response(),
+                &mut cache
+            )
+            .unwrap(),
             CaptureResult {
                 name: "duration".to_string(),
                 value: Value::Number(Number::from(1.5)),
