@@ -17,37 +17,38 @@
  */
 
 use chrono::Utc;
-
 use hurl_core::ast::SourceInfo;
 
-use crate::runner::{Error, Number, RunnerError, Value};
+use crate::runner::{Number, RunnerError, RunnerErrorKind, Value};
 
 pub fn eval_days_after_now(
     value: &Value,
     source_info: SourceInfo,
     assert: bool,
-) -> Result<Option<Value>, Error> {
+) -> Result<Option<Value>, RunnerError> {
     match value {
         Value::Date(value) => {
             let diff = value.signed_duration_since(Utc::now());
             Ok(Some(Value::Number(Number::Integer(diff.num_days()))))
         }
         v => {
-            let inner = RunnerError::FilterInvalidInput(v._type());
-            Err(Error::new(source_info, inner, assert))
+            let kind = RunnerErrorKind::FilterInvalidInput(v._type());
+            Err(RunnerError::new(source_info, kind, assert))
         }
     }
 }
 
 #[cfg(test)]
 pub mod tests {
-    use crate::runner::filter::eval::eval_filter;
-    use chrono::offset::Utc;
-    use chrono::Duration;
-    use hurl_core::ast::{Filter, FilterValue, Pos, SourceInfo};
     use std::collections::HashMap;
 
+    use chrono::offset::Utc;
+    use chrono::Duration;
+    use hurl_core::ast::{Filter, FilterValue, SourceInfo};
+    use hurl_core::reader::Pos;
+
     use super::*;
+    use crate::runner::filter::eval::eval_filter;
 
     #[test]
     pub fn eval_filter_days_after_before_now() {
