@@ -20,8 +20,7 @@ use std::collections::HashMap;
 use hurl_core::ast::{RegexValue, SourceInfo};
 
 use crate::runner::regex::eval_regex_value;
-
-use crate::runner::{Error, RunnerError, Value};
+use crate::runner::{RunnerError, RunnerErrorKind, Value};
 
 pub fn eval_regex(
     value: &Value,
@@ -29,7 +28,7 @@ pub fn eval_regex(
     variables: &HashMap<String, Value>,
     source_info: SourceInfo,
     assert: bool,
-) -> Result<Option<Value>, Error> {
+) -> Result<Option<Value>, RunnerError> {
     let re = eval_regex_value(regex_value, variables)?;
     match value {
         Value::String(s) => match re.captures(s.as_str()) {
@@ -40,8 +39,8 @@ pub fn eval_regex(
             None => Ok(None),
         },
         v => {
-            let inner = RunnerError::FilterInvalidInput(v._type());
-            Err(Error::new(source_info, inner, assert))
+            let kind = RunnerErrorKind::FilterInvalidInput(v._type());
+            Err(RunnerError::new(source_info, kind, assert))
         }
     }
 }
@@ -50,11 +49,14 @@ pub fn eval_regex(
 pub mod tests {
     use std::collections::HashMap;
 
-    use crate::runner::filter::eval::eval_filter;
-    use crate::runner::{RunnerError, Value};
     use hurl_core::ast::{
-        Filter, FilterValue, Pos, RegexValue, SourceInfo, Template, TemplateElement, Whitespace,
+        Filter, FilterValue, RegexValue, SourceInfo, Template, TemplateElement, Whitespace,
     };
+    use hurl_core::reader::Pos;
+
+    use crate::runner::filter::eval::eval_filter;
+    use crate::runner::{RunnerErrorKind, Value};
+
     #[test]
     fn eval_filter_regex() {
         // regex "Hello (.*)!"
@@ -97,8 +99,8 @@ pub mod tests {
             SourceInfo::new(Pos::new(1, 1), Pos::new(1, 20))
         );
         assert_eq!(
-            error.inner,
-            RunnerError::FilterInvalidInput("boolean".to_string())
+            error.kind,
+            RunnerErrorKind::FilterInvalidInput("boolean".to_string())
         );
     }
 
@@ -135,6 +137,6 @@ pub mod tests {
             error.source_info,
             SourceInfo::new(Pos::new(1, 7), Pos::new(1, 20))
         );
-        assert_eq!(error.inner, RunnerError::InvalidRegex);
+        assert_eq!(error.kind, RunnerErrorKind::InvalidRegex);
     }
 }
