@@ -18,26 +18,26 @@
 
 use hurl_core::ast::SourceInfo;
 
-use crate::runner::{Error, Number, RunnerError, Value};
+use crate::runner::{Number, RunnerError, RunnerErrorKind, Value};
 
 pub fn eval_to_int(
     value: &Value,
     source_info: SourceInfo,
     assert: bool,
-) -> Result<Option<Value>, Error> {
+) -> Result<Option<Value>, RunnerError> {
     match value {
         Value::Number(Number::Integer(v)) => Ok(Some(Value::Number(Number::Integer(*v)))),
         Value::Number(Number::Float(v)) => Ok(Some(Value::Number(Number::Integer(*v as i64)))),
         Value::String(v) => match v.parse::<i64>() {
             Ok(i) => Ok(Some(Value::Number(Number::Integer(i)))),
             _ => {
-                let inner = RunnerError::FilterInvalidInput(value.display());
-                Err(Error::new(source_info, inner, assert))
+                let kind = RunnerErrorKind::FilterInvalidInput(value.display());
+                Err(RunnerError::new(source_info, kind, assert))
             }
         },
         v => {
-            let inner = RunnerError::FilterInvalidInput(v.display());
-            Err(Error::new(source_info, inner, assert))
+            let kind = RunnerErrorKind::FilterInvalidInput(v.display());
+            Err(RunnerError::new(source_info, kind, assert))
         }
     }
 }
@@ -45,11 +45,14 @@ pub fn eval_to_int(
 #[cfg(test)]
 pub mod tests {
 
-    use crate::runner::filter::eval::eval_filter;
-    use crate::runner::{Number, RunnerError, Value};
-    use hurl_core::ast::{Filter, FilterValue, Pos, SourceInfo};
     use std::collections::HashMap;
 
+    use hurl_core::ast::{Filter, FilterValue, SourceInfo};
+    use hurl_core::reader::Pos;
+
+    use crate::runner::filter::eval::eval_filter;
+    use crate::runner::{Number, RunnerErrorKind, Value};
+
     #[test]
     pub fn eval_filter_to_int() {
         let variables = HashMap::new();
@@ -108,15 +111,15 @@ pub mod tests {
         .err()
         .unwrap();
         assert_eq!(
-            err.inner,
-            RunnerError::FilterInvalidInput("string <123x>".to_string())
+            err.kind,
+            RunnerErrorKind::FilterInvalidInput("string <123x>".to_string())
         );
         let err = eval_filter(&filter, &Value::Bool(true), &variables, false)
             .err()
             .unwrap();
         assert_eq!(
-            err.inner,
-            RunnerError::FilterInvalidInput("bool <true>".to_string())
+            err.kind,
+            RunnerErrorKind::FilterInvalidInput("bool <true>".to_string())
         );
     }
 }
