@@ -20,7 +20,8 @@ use std::collections::HashMap;
 use hurl_core::ast::{SourceInfo, Template};
 
 use crate::runner::template::eval_template;
-use crate::runner::{xpath, Error, RunnerError, Value};
+use crate::runner::xpath::{Document, Format, XPathError};
+use crate::runner::{RunnerError, RunnerErrorKind, Value};
 
 pub fn eval_xpath(
     value: &Value,
@@ -28,51 +29,42 @@ pub fn eval_xpath(
     variables: &HashMap<String, Value>,
     source_info: SourceInfo,
     assert: bool,
-) -> Result<Option<Value>, Error> {
+) -> Result<Option<Value>, RunnerError> {
     match value {
         Value::String(xml) => {
             // The filter will use the HTML parser that should also work with XML input
-            let is_html = true;
-            eval_xpath_string(xml, expr, variables, source_info, is_html)
+            let Ok(doc) = Document::parse(xml, Format::Html) else {
+                return Err(RunnerError::new(
+                    source_info,
+                    RunnerErrorKind::QueryInvalidXml,
+                    false,
+                ));
+            };
+            eval_xpath_doc(&doc, expr, variables)
         }
         v => {
-            let inner = RunnerError::FilterInvalidInput(v._type());
-            Err(Error::new(source_info, inner, assert))
+            let kind = RunnerErrorKind::FilterInvalidInput(v._type());
+            Err(RunnerError::new(source_info, kind, assert))
         }
     }
 }
 
-pub fn eval_xpath_string(
-    xml: &str,
-    expr_template: &Template,
+pub fn eval_xpath_doc(
+    doc: &Document,
+    expr: &Template,
     variables: &HashMap<String, Value>,
-    source_info: SourceInfo,
-    is_html: bool,
-) -> Result<Option<Value>, Error> {
-    let expr = eval_template(expr_template, variables)?;
-    let result = if is_html {
-        xpath::eval_html(xml, &expr)
-    } else {
-        xpath::eval_xml(xml, &expr)
-    };
+) -> Result<Option<Value>, RunnerError> {
+    let expr_str = eval_template(expr, variables)?;
+    let result = doc.eval_xpath(&expr_str);
     match result {
         Ok(value) => Ok(Some(value)),
-        Err(xpath::XpathError::InvalidXml) => {
-            Err(Error::new(source_info, RunnerError::QueryInvalidXml, false))
-        }
-        Err(xpath::XpathError::InvalidHtml) => {
-            Err(Error::new(source_info, RunnerError::QueryInvalidXml, false))
-        }
-        Err(xpath::XpathError::Eval) => Err(Error::new(
-            expr_template.source_info,
-            RunnerError::QueryInvalidXpathEval,
+        Err(XPathError::Eval) => Err(RunnerError::new(
+            expr.source_info,
+            RunnerErrorKind::QueryInvalidXpathEval,
             false,
         )),
-        Err(xpath::XpathError::Unsupported) => {
+        Err(XPathError::Unsupported) => {
             panic!("Unsupported xpath {expr}"); // good usecase for panic - I could not reproduce this usecase myself
         }
     }
 }
-
-#[cfg(test)]
-pub mod tests {}
