@@ -18,9 +18,10 @@
 use std::collections::HashMap;
 
 use hurl_core::ast::{JsonListElement, JsonObjectElement, JsonValue, Template, TemplateElement};
-use hurl_core::parser::{parse_json_boolean, parse_json_null, parse_json_number, Reader};
+use hurl_core::parser::{parse_json_boolean, parse_json_null, parse_json_number};
+use hurl_core::reader::Reader;
 
-use crate::runner::error::{Error, RunnerError};
+use crate::runner::error::{RunnerError, RunnerErrorKind};
 use crate::runner::template::render_expression;
 use crate::runner::value::Value;
 
@@ -31,7 +32,7 @@ pub fn eval_json_value(
     json_value: &JsonValue,
     variables: &HashMap<String, Value>,
     keep_whitespace: bool,
-) -> Result<String, Error> {
+) -> Result<String, RunnerError> {
     match json_value {
         JsonValue::Null => Ok("null".to_string()),
         JsonValue::Number(s) => Ok(s.clone()),
@@ -70,20 +71,20 @@ pub fn eval_json_value(
             // The String can only be null, a bool, a number
             // It will be easier when your variables value have a type
             let mut reader = Reader::new(s.as_str());
-            let start = reader.state;
+            let start = reader.cursor();
             if parse_json_number(&mut reader).is_ok() {
                 return Ok(s);
             }
-            reader.state = start;
+            reader.seek(start);
             if parse_json_boolean(&mut reader).is_ok() {
                 return Ok(s);
             }
-            reader.state = start;
+            reader.seek(start);
             if parse_json_null(&mut reader).is_ok() {
                 return Ok(s);
             }
-            let inner = RunnerError::InvalidJson { value: s };
-            Err(Error::new(exp.variable.source_info, inner, false))
+            let kind = RunnerErrorKind::InvalidJson { value: s };
+            Err(RunnerError::new(exp.variable.source_info, kind, false))
         }
     }
 }
@@ -95,7 +96,7 @@ fn eval_json_list_element(
     element: &JsonListElement,
     variables: &HashMap<String, Value>,
     keep_whitespace: bool,
-) -> Result<String, Error> {
+) -> Result<String, RunnerError> {
     let s = eval_json_value(&element.value, variables, keep_whitespace)?;
     if keep_whitespace {
         Ok(format!("{}{}{}", element.space0, s, element.space1))
@@ -111,7 +112,7 @@ fn eval_json_object_element(
     element: &JsonObjectElement,
     variables: &HashMap<String, Value>,
     keep_whitespace: bool,
-) -> Result<String, Error> {
+) -> Result<String, RunnerError> {
     let value = eval_json_value(&element.value, variables, keep_whitespace)?;
     if keep_whitespace {
         Ok(format!(
@@ -132,7 +133,7 @@ fn eval_json_object_element(
 pub fn eval_json_template(
     template: &Template,
     variables: &HashMap<String, Value>,
-) -> Result<String, Error> {
+) -> Result<String, RunnerError> {
     let Template { elements, .. } = template;
     {
         let mut value = String::new();
@@ -149,7 +150,7 @@ pub fn eval_json_template(
 fn eval_json_template_element(
     template_element: &TemplateElement,
     variables: &HashMap<String, Value>,
-) -> Result<String, Error> {
+) -> Result<String, RunnerError> {
     match template_element {
         TemplateElement::String { encoded, .. } => Ok(encoded.clone()),
         TemplateElement::Expression(expr) => {
@@ -177,8 +178,9 @@ fn encode_json_char(c: char) -> String {
 #[cfg(test)]
 mod tests {
     use hurl_core::ast::*;
+    use hurl_core::reader::Pos;
 
-    use super::super::error::RunnerError;
+    use super::super::error::RunnerErrorKind;
     use super::*;
 
     pub fn json_hello_world_value() -> JsonValue {
@@ -274,8 +276,8 @@ mod tests {
             SourceInfo::new(Pos::new(1, 15), Pos::new(1, 19))
         );
         assert_eq!(
-            error.inner,
-            RunnerError::TemplateVariableNotDefined {
+            error.kind,
+            RunnerErrorKind::TemplateVariableNotDefined {
                 name: "name".to_string()
             }
         );
