@@ -16,16 +16,16 @@
  *
  */
 use std::collections::HashMap;
-use std::time::Duration;
 
 use hurl_core::ast::{
-    BooleanOption, Entry, EntryOption, Float, NaturalOption, Number as AstNumber, OptionKind,
-    Retry, RetryOption, SectionValue, VariableDefinition, VariableValue,
+    BooleanOption, CountOption, DurationOption, Entry, EntryOption, Float, Number as AstNumber,
+    OptionKind, SectionValue, VariableDefinition, VariableValue,
 };
+use hurl_core::typing::{Count, DurationUnit};
 
 use crate::http::{IpResolve, RequestedHttpVersion};
 use crate::runner::template::{eval_expression, eval_template};
-use crate::runner::{Error, Number, Output, RunnerError, RunnerOptions, Value};
+use crate::runner::{Number, Output, RunnerError, RunnerErrorKind, RunnerOptions, Value};
 use crate::util::logger::{Logger, Verbosity};
 
 /// Returns a new [`RunnerOptions`] based on the `entry` optional Options section
@@ -36,17 +36,17 @@ pub fn get_entry_options(
     runner_options: &RunnerOptions,
     variables: &mut HashMap<String, Value>,
     logger: &mut Logger,
-) -> Result<RunnerOptions, Error> {
+) -> Result<RunnerOptions, RunnerError> {
     let runner_options = runner_options.clone();
     // When used globally (on the command line), `--output` writes the last successful request
     // to `output` file. We don't want to output every entry's response, so we initialise
     // output to `None`.
-    let mut runner_options = RunnerOptions {
+    let mut entry_options = RunnerOptions {
         output: None,
         ..runner_options
     };
     if !has_options(entry) {
-        return Ok(runner_options);
+        return Ok(entry_options);
     }
 
     logger.debug("");
@@ -58,31 +58,32 @@ pub fn get_entry_options(
                 match &option.kind {
                     OptionKind::AwsSigV4(value) => {
                         let value = eval_template(value, variables)?;
-                        runner_options.aws_sigv4 = Some(value);
+                        entry_options.aws_sigv4 = Some(value);
                     }
                     OptionKind::CaCertificate(filename) => {
                         let value = eval_template(filename, variables)?;
-                        runner_options.cacert_file = Some(value);
+                        entry_options.cacert_file = Some(value);
                     }
                     OptionKind::ClientCert(filename) => {
                         let value = eval_template(filename, variables)?;
-                        runner_options.client_cert_file = Some(value);
+                        entry_options.client_cert_file = Some(value);
                     }
                     OptionKind::ClientKey(filename) => {
                         let value = eval_template(filename, variables)?;
-                        runner_options.client_key_file = Some(value);
+                        entry_options.client_key_file = Some(value);
                     }
                     OptionKind::Compressed(value) => {
                         let value = eval_boolean_option(value, variables)?;
-                        runner_options.compressed = value;
+                        entry_options.compressed = value;
                     }
                     OptionKind::ConnectTo(value) => {
                         let value = eval_template(value, variables)?;
-                        runner_options.connects_to.push(value);
+                        entry_options.connects_to.push(value);
                     }
                     OptionKind::Delay(value) => {
-                        let value = eval_natural_option(value, variables)?;
-                        runner_options.delay = Duration::from_millis(value);
+                        let value =
+                            eval_duration_option(value, variables, DurationUnit::MilliSecond)?;
+                        entry_options.delay = value;
                     }
                     // HTTP version options (such as http1.0, http1.1, http2 etc...) are activated
                     // through a flag. In an `[Options]` section, the signification of such a flag is:
@@ -111,51 +112,51 @@ pub fn get_entry_options(
                     OptionKind::Http10(value) => {
                         let value = eval_boolean_option(value, variables)?;
                         if value {
-                            runner_options.http_version = RequestedHttpVersion::Http10;
+                            entry_options.http_version = RequestedHttpVersion::Http10;
                         }
                     }
                     OptionKind::Http11(value) => {
                         let value = eval_boolean_option(value, variables)?;
                         if value {
-                            runner_options.http_version = RequestedHttpVersion::Http11;
+                            entry_options.http_version = RequestedHttpVersion::Http11;
                         } else {
-                            runner_options.http_version = RequestedHttpVersion::Http10;
+                            entry_options.http_version = RequestedHttpVersion::Http10;
                         }
                     }
                     OptionKind::Http2(value) => {
                         let value = eval_boolean_option(value, variables)?;
                         if value {
-                            runner_options.http_version = RequestedHttpVersion::Http2;
+                            entry_options.http_version = RequestedHttpVersion::Http2;
                         } else {
-                            runner_options.http_version = RequestedHttpVersion::Http11;
+                            entry_options.http_version = RequestedHttpVersion::Http11;
                         }
                     }
                     OptionKind::Http3(value) => {
                         let value = eval_boolean_option(value, variables)?;
                         if value {
-                            runner_options.http_version = RequestedHttpVersion::Http3;
+                            entry_options.http_version = RequestedHttpVersion::Http3;
                         } else {
-                            runner_options.http_version = RequestedHttpVersion::Http2;
+                            entry_options.http_version = RequestedHttpVersion::Http2;
                         }
                     }
                     OptionKind::FollowLocation(value) => {
                         let value = eval_boolean_option(value, variables)?;
-                        runner_options.follow_location = value;
+                        entry_options.follow_location = value;
                     }
                     OptionKind::FollowLocationTrusted(value) => {
                         let value = eval_boolean_option(value, variables)?;
                         if value {
-                            runner_options.follow_location = true;
+                            entry_options.follow_location = true;
                         }
-                        runner_options.follow_location_trusted = value;
+                        entry_options.follow_location_trusted = value;
                     }
                     OptionKind::Insecure(value) => {
                         let value = eval_boolean_option(value, variables)?;
-                        runner_options.insecure = value;
+                        entry_options.insecure = value;
                     }
                     OptionKind::IpV4(value) => {
                         let value = eval_boolean_option(value, variables)?;
-                        runner_options.ip_resolve = if value {
+                        entry_options.ip_resolve = if value {
                             IpResolve::IpV4
                         } else {
                             IpResolve::IpV6
@@ -163,64 +164,69 @@ pub fn get_entry_options(
                     }
                     OptionKind::IpV6(value) => {
                         let value = eval_boolean_option(value, variables)?;
-                        runner_options.ip_resolve = if value {
+                        entry_options.ip_resolve = if value {
                             IpResolve::IpV6
                         } else {
                             IpResolve::IpV4
                         }
                     }
                     OptionKind::MaxRedirect(value) => {
-                        let value = eval_natural_option(value, variables)?;
-                        runner_options.max_redirect = Some(value as usize);
+                        let value = eval_count_option(value, variables)?;
+                        entry_options.max_redirect = value;
                     }
                     OptionKind::NetRc(value) => {
                         let value = eval_boolean_option(value, variables)?;
-                        runner_options.netrc = value;
+                        entry_options.netrc = value;
                     }
                     OptionKind::NetRcFile(value) => {
                         let filename = eval_template(value, variables)?;
-                        runner_options.netrc_file = Some(filename);
+                        entry_options.netrc_file = Some(filename);
                     }
                     OptionKind::NetRcOptional(value) => {
                         let value = eval_boolean_option(value, variables)?;
-                        runner_options.netrc_optional = value;
+                        entry_options.netrc_optional = value;
                     }
                     OptionKind::Output(output) => {
                         let filename = eval_template(output, variables)?;
                         let output = Output::new(&filename);
-                        runner_options.output = Some(output);
+                        entry_options.output = Some(output);
                     }
                     OptionKind::PathAsIs(value) => {
                         let value = eval_boolean_option(value, variables)?;
-                        runner_options.path_as_is = value;
+                        entry_options.path_as_is = value;
                     }
                     OptionKind::Proxy(value) => {
                         let value = eval_template(value, variables)?;
-                        runner_options.proxy = Some(value);
+                        entry_options.proxy = Some(value);
+                    }
+                    OptionKind::Repeat(value) => {
+                        let value = eval_count_option(value, variables)?;
+                        entry_options.repeat = Some(value);
                     }
                     OptionKind::Resolve(value) => {
                         let value = eval_template(value, variables)?;
-                        runner_options.resolves.push(value);
+                        entry_options.resolves.push(value);
                     }
                     OptionKind::Retry(value) => {
-                        let value = eval_retry_option(value, variables)?;
-                        runner_options.retry = value;
+                        let value = eval_count_option(value, variables)?;
+                        entry_options.retry = Some(value);
                     }
                     OptionKind::RetryInterval(value) => {
-                        let value = eval_natural_option(value, variables)?;
-                        runner_options.retry_interval = Duration::from_millis(value);
+                        let value =
+                            eval_duration_option(value, variables, DurationUnit::MilliSecond)?;
+                        entry_options.retry_interval = value;
                     }
                     OptionKind::Skip(value) => {
                         let value = eval_boolean_option(value, variables)?;
-                        runner_options.skip = value;
+                        entry_options.skip = value;
                     }
                     OptionKind::UnixSocket(value) => {
                         let value = eval_template(value, variables)?;
-                        runner_options.unix_socket = Some(value);
+                        entry_options.unix_socket = Some(value);
                     }
                     OptionKind::User(value) => {
                         let value = eval_template(value, variables)?;
-                        runner_options.user = Some(value);
+                        entry_options.user = Some(value);
                     }
                     OptionKind::Variable(VariableDefinition { name, value, .. }) => {
                         let value = eval_variable_value(value, variables)?;
@@ -240,7 +246,7 @@ pub fn get_entry_options(
             }
         }
     }
-    Ok(runner_options)
+    Ok(entry_options)
 }
 
 /// Logs an entry option.
@@ -264,7 +270,7 @@ pub fn get_entry_verbosity(
     entry: &Entry,
     default_verbosity: Option<Verbosity>,
     variables: &HashMap<String, Value>,
-) -> Result<Option<Verbosity>, Error> {
+) -> Result<Option<Verbosity>, RunnerError> {
     let mut verbosity = default_verbosity;
 
     for section in &entry.request.sections {
@@ -298,93 +304,106 @@ pub fn get_entry_verbosity(
 fn eval_boolean_option(
     boolean_value: &BooleanOption,
     variables: &HashMap<String, Value>,
-) -> Result<bool, Error> {
+) -> Result<bool, RunnerError> {
     match boolean_value {
         BooleanOption::Literal(value) => Ok(*value),
         BooleanOption::Expression(expr) => match eval_expression(expr, variables)? {
             Value::Bool(value) => Ok(value),
             v => {
-                let inner = RunnerError::TemplateVariableInvalidType {
+                let kind = RunnerErrorKind::TemplateVariableInvalidType {
                     name: expr.variable.name.clone(),
-                    value: v.to_string(),
+                    value: v.format(),
                     expecting: "boolean".to_string(),
                 };
-                Err(Error::new(expr.variable.source_info, inner, false))
+                Err(RunnerError::new(expr.variable.source_info, kind, false))
             }
         },
     }
 }
 
-fn eval_natural_option(
-    natural_value: &NaturalOption,
+fn eval_count_option(
+    count_value: &CountOption,
     variables: &HashMap<String, Value>,
-) -> Result<u64, Error> {
-    match natural_value {
-        NaturalOption::Literal(value) => Ok(*value),
-        NaturalOption::Expression(expr) => match eval_expression(expr, variables)? {
-            Value::Number(Number::Integer(value)) => {
-                if value < 0 {
-                    let inner = RunnerError::TemplateVariableInvalidType {
-                        name: expr.variable.name.clone(),
-                        value: value.to_string(),
-                        expecting: "positive integer".to_string(),
-                    };
-                    Err(Error::new(expr.variable.source_info, inner, false))
-                } else {
-                    Ok(value as u64)
-                }
-            }
-            v => {
-                let inner = RunnerError::TemplateVariableInvalidType {
-                    name: expr.variable.name.clone(),
-                    value: v.to_string(),
-                    expecting: "positive integer".to_string(),
-                };
-                Err(Error::new(expr.variable.source_info, inner, false))
-            }
-        },
-    }
-}
-
-fn eval_retry_option(
-    retry_option_value: &RetryOption,
-    variables: &HashMap<String, Value>,
-) -> Result<Retry, Error> {
-    match retry_option_value {
-        RetryOption::Literal(retry) => Ok(*retry),
-        RetryOption::Expression(expr) => match eval_expression(expr, variables)? {
+) -> Result<Count, RunnerError> {
+    match count_value {
+        CountOption::Literal(repeat) => Ok(*repeat),
+        CountOption::Expression(expr) => match eval_expression(expr, variables)? {
             Value::Number(Number::Integer(value)) => {
                 if value == -1 {
-                    Ok(Retry::Infinite)
-                } else if value == 0 {
-                    Ok(Retry::None)
-                } else if value > 0 {
-                    Ok(Retry::Finite(value as usize))
+                    Ok(Count::Infinite)
+                } else if value >= 0 {
+                    Ok(Count::Finite(value as usize))
                 } else {
-                    let inner = RunnerError::TemplateVariableInvalidType {
+                    let kind = RunnerErrorKind::TemplateVariableInvalidType {
                         name: expr.variable.name.clone(),
-                        value: value.to_string(),
-                        expecting: "integer".to_string(),
+                        value: format!("integer <{value}>"),
+                        expecting: "integer >= -1".to_string(),
                     };
-                    Err(Error::new(expr.variable.source_info, inner, false))
+                    Err(RunnerError::new(expr.variable.source_info, kind, false))
                 }
             }
             v => {
-                let inner = RunnerError::TemplateVariableInvalidType {
+                let kind = RunnerErrorKind::TemplateVariableInvalidType {
                     name: expr.variable.name.clone(),
-                    value: v.to_string(),
+                    value: v.format(),
                     expecting: "integer".to_string(),
                 };
-                Err(Error::new(expr.variable.source_info, inner, false))
+                Err(RunnerError::new(expr.variable.source_info, kind, false))
             }
         },
     }
 }
 
+/// return duration value in milliseconds
+fn eval_duration_option(
+    duration_value: &DurationOption,
+    variables: &HashMap<String, Value>,
+    default_unit: DurationUnit,
+) -> Result<std::time::Duration, RunnerError> {
+    let millis = match duration_value {
+        DurationOption::Literal(value) => {
+            let unit = value.unit.unwrap_or(default_unit);
+
+            match unit {
+                DurationUnit::MilliSecond => value.value,
+                DurationUnit::Second => value.value * 1000,
+                DurationUnit::Minute => value.value * 1000 * 60,
+            }
+        }
+        DurationOption::Expression(expr) => match eval_expression(expr, variables)? {
+            Value::Number(Number::Integer(value)) => {
+                if value < 0 {
+                    let kind = RunnerErrorKind::TemplateVariableInvalidType {
+                        name: expr.variable.name.clone(),
+                        value: format!("integer <{value}>"),
+                        expecting: "positive integer".to_string(),
+                    };
+                    return Err(RunnerError::new(expr.variable.source_info, kind, false));
+                } else {
+                    match default_unit {
+                        DurationUnit::MilliSecond => value as u64,
+                        DurationUnit::Second => (value * 1000) as u64,
+                        DurationUnit::Minute => (value * 1000 * 60) as u64,
+                    }
+                }
+            }
+            v => {
+                let kind = RunnerErrorKind::TemplateVariableInvalidType {
+                    name: expr.variable.name.clone(),
+                    value: v.format(),
+                    expecting: "positive integer".to_string(),
+                };
+                return Err(RunnerError::new(expr.variable.source_info, kind, false));
+            }
+        },
+    };
+    Ok(std::time::Duration::from_millis(millis))
+}
+
 fn eval_variable_value(
     variable_value: &VariableValue,
     variables: &mut HashMap<String, Value>,
-) -> Result<Value, Error> {
+) -> Result<Value, RunnerError> {
     match variable_value {
         VariableValue::Null => Ok(Value::Null),
         VariableValue::Bool(v) => Ok(Value::Bool(*v)),
@@ -406,16 +425,18 @@ fn eval_number(number: &AstNumber) -> Value {
 
 #[cfg(test)]
 mod tests {
-    use hurl_core::ast::{Expr, Pos, SourceInfo, Variable, Whitespace};
+    use hurl_core::ast::{Expr, SourceInfo, Variable, Whitespace};
+    use hurl_core::reader::Pos;
+    use hurl_core::typing::{Duration, DurationUnit};
 
     use super::*;
-    use crate::runner::RunnerError;
+    use crate::runner::RunnerErrorKind;
 
     fn verbose_option_template() -> BooleanOption {
         // {{verbose}}
         BooleanOption::Expression(Expr {
             space0: Whitespace {
-                value: "".to_string(),
+                value: String::new(),
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             variable: Variable {
@@ -423,17 +444,17 @@ mod tests {
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             space1: Whitespace {
-                value: "".to_string(),
+                value: String::new(),
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
         })
     }
 
-    fn retry_option_template() -> NaturalOption {
+    fn retry_option_template() -> DurationOption {
         // {{retry}}
-        NaturalOption::Expression(Expr {
+        DurationOption::Expression(Expr {
             space0: Whitespace {
-                value: "".to_string(),
+                value: String::new(),
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             variable: Variable {
@@ -441,7 +462,7 @@ mod tests {
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             space1: Whitespace {
-                value: "".to_string(),
+                value: String::new(),
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
         })
@@ -464,8 +485,8 @@ mod tests {
             .unwrap();
         assert!(!error.assert);
         assert_eq!(
-            error.inner,
-            RunnerError::TemplateVariableNotDefined {
+            error.kind,
+            RunnerErrorKind::TemplateVariableNotDefined {
                 name: "verbose".to_string()
             }
         );
@@ -475,10 +496,10 @@ mod tests {
             .err()
             .unwrap();
         assert_eq!(
-            error.inner,
-            RunnerError::TemplateVariableInvalidType {
+            error.kind,
+            RunnerErrorKind::TemplateVariableInvalidType {
                 name: "verbose".to_string(),
-                value: "10".to_string(),
+                value: "integer <10>".to_string(),
                 expecting: "boolean".to_string()
             }
         );
@@ -488,14 +509,24 @@ mod tests {
     fn test_eval_natural_option() {
         let mut variables = HashMap::default();
         assert_eq!(
-            eval_natural_option(&NaturalOption::Literal(1), &variables).unwrap(),
-            1
+            eval_duration_option(
+                &DurationOption::Literal(Duration::new(1, Some(DurationUnit::Second))),
+                &variables,
+                DurationUnit::MilliSecond
+            )
+            .unwrap(),
+            std::time::Duration::from_millis(1000)
         );
 
         variables.insert("retry".to_string(), Value::Number(Number::Integer(10)));
         assert_eq!(
-            eval_natural_option(&retry_option_template(), &variables).unwrap(),
-            10
+            eval_duration_option(
+                &retry_option_template(),
+                &variables,
+                DurationUnit::MilliSecond
+            )
+            .unwrap(),
+            std::time::Duration::from_millis(10)
         );
     }
 }
