@@ -15,15 +15,16 @@
  * limitations under the License.
  *
  */
-use std::fmt;
-use std::fs::File;
+use std::fs::{File, OpenOptions};
 use std::io::Write;
 use std::path::{Path, PathBuf};
+use std::{fmt, io};
 
-use crate::runner::{Error, RunnerError};
+use hurl_core::ast::SourceInfo;
+
+use crate::runner::{RunnerError, RunnerErrorKind};
 use crate::util::path::ContextDir;
 use crate::util::term::Stdout;
-use hurl_core::ast::{Pos, SourceInfo};
 
 /// Represents the output of write operation: can be either a file or standard output.
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -54,68 +55,95 @@ impl Output {
         }
     }
 
+    /// Writes these `bytes` to the output.
+    ///
+    /// If output is a standard output variant, `stdout` is used to write the bytes. If `append`
+    /// the output file is created in append mode, else any existing file will be truncated.
+    pub fn write(&self, bytes: &[u8], stdout: &mut Stdout, append: bool) -> Result<(), io::Error> {
+        match self {
+            Output::Stdout => stdout.write_all(bytes)?,
+            Output::File(filename) => {
+                let mut file = OpenOptions::new()
+                    .write(true)
+                    .append(append)
+                    .open(filename)?;
+                file.write_all(bytes)?;
+            }
+        }
+        Ok(())
+    }
+
     /// Writes these `bytes` to the output.
     ///
     /// If output is a standard output variant, `stdout` is used to write the bytes.
-    /// If output is a file variant, an optional `context_dir` can be used to check authorized
-    /// write access.
-    pub fn write(
+    /// If output is a file variant, `context_dir` is used to check authorized write access.
+    pub fn write_with_context_dir(
         &self,
         bytes: &[u8],
         stdout: &mut Stdout,
-        context_dir: Option<&ContextDir>,
-    ) -> Result<(), Error> {
+        context_dir: &ContextDir,
+        source_info: SourceInfo,
+    ) -> Result<(), RunnerError> {
+        // TODO: Check if write method above can be reused
         match self {
             Output::Stdout => match stdout.write_all(bytes) {
                 Ok(_) => Ok(()),
                 Err(e) => {
                     let filename = Path::new("stdout");
-                    Err(Error::new_file_write_access(filename, &e.to_string()))
+                    Err(RunnerError::new_file_write_access(
+                        filename,
+                        &e.to_string(),
+                        source_info,
+                    ))
                 }
             },
             Output::File(filename) => {
-                // If we have a context dir, we check if we can write to this filename and compute
-                // the new filename given this context dir.
-                let filename = match context_dir {
-                    None => filename.clone(),
-                    Some(context_dir) => {
-                        if !context_dir.is_access_allowed(filename) {
-                            return Err(Error::new_unauthorized_file_access(filename));
-                        }
-                        context_dir.resolved_path(filename)
-                    }
-                };
+                if !context_dir.is_access_allowed(filename) {
+                    return Err(RunnerError::new_unauthorized_file_access(
+                        filename,
+                        source_info,
+                    ));
+                }
+                // we check if we can write to this filename and compute the new filename given this context dir.
+                let filename = context_dir.resolved_path(filename);
                 let mut file = match File::create(&filename) {
                     Ok(file) => file,
-                    Err(e) => return Err(Error::new_file_write_access(&filename, &e.to_string())),
+                    Err(e) => {
+                        return Err(RunnerError::new_file_write_access(
+                            &filename,
+                            &e.to_string(),
+                            source_info,
+                        ))
+                    }
                 };
                 match file.write_all(bytes) {
                     Ok(_) => Ok(()),
-                    Err(e) => Err(Error::new_file_write_access(&filename, &e.to_string())),
+                    Err(e) => Err(RunnerError::new_file_write_access(
+                        &filename,
+                        &e.to_string(),
+                        source_info,
+                    )),
                 }
             }
         }
     }
 }
 
-// TODO: improve the error with a [`SourceInfo`] passed in parameter.
-impl Error {
+impl RunnerError {
     /// Creates a new file write access error.
-    fn new_file_write_access(path: &Path, error: &str) -> Error {
-        let source_info = SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0));
+    fn new_file_write_access(path: &Path, error: &str, source_info: SourceInfo) -> RunnerError {
         let path = path.to_path_buf();
-        let inner = RunnerError::FileWriteAccess {
+        let kind = RunnerErrorKind::FileWriteAccess {
             path,
             error: error.to_string(),
         };
-        Error::new(source_info, inner, false)
+        RunnerError::new(source_info, kind, false)
     }
 
     /// Creates a new authorization access error.
-    fn new_unauthorized_file_access(path: &Path) -> Error {
-        let source_info = SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0));
+    fn new_unauthorized_file_access(path: &Path, source_info: SourceInfo) -> RunnerError {
         let path = path.to_path_buf();
-        let inner = RunnerError::UnauthorizedFileAccess { path };
-        Error::new(source_info, inner, false)
+        let kind = RunnerErrorKind::UnauthorizedFileAccess { path };
+        RunnerError::new(source_info, kind, false)
     }
 }
