@@ -20,7 +20,8 @@ use std::collections::HashMap;
 use hurl_core::ast::*;
 
 use crate::http;
-use crate::runner::error::{Error, RunnerError};
+use crate::runner::cache::BodyCache;
+use crate::runner::error::{RunnerError, RunnerErrorKind};
 use crate::runner::result::{AssertResult, CaptureResult};
 use crate::runner::{assert, body, capture, json, multiline, template, Value};
 use crate::util::path::ContextDir;
@@ -55,10 +56,14 @@ pub fn eval_version_status_asserts(
 ///
 /// Asserts on status and version and not run in this function, there are run with `eval_version_status_asserts`
 /// as they're semantically stronger.
+///
+/// The `cache` is used to store XML / JSON structured response data and avoid redundant parsing
+/// operation on the response.
 pub fn eval_asserts(
     response: &Response,
     variables: &HashMap<String, Value>,
     http_response: &http::Response,
+    cache: &mut BodyCache,
     context_dir: &ContextDir,
 ) -> Vec<AssertResult> {
     let mut asserts = vec![];
@@ -80,9 +85,9 @@ pub fn eval_asserts(
                         let actuals = http_response.headers.values(&header_name);
                         if actuals.is_empty() {
                             let result = AssertResult::Header {
-                                actual: Err(Error::new(
+                                actual: Err(RunnerError::new(
                                     header.key.source_info,
-                                    RunnerError::QueryHeaderNotFound,
+                                    RunnerErrorKind::QueryHeaderNotFound,
                                     false,
                                 )),
                                 expected,
@@ -143,9 +148,9 @@ pub fn eval_asserts(
     }
 
     // Then, checks all the explicit asserts.
-    for assert in &response.asserts() {
+    for assert in response.asserts() {
         let assert_result =
-            assert::eval_explicit_assert(assert, variables, http_response, context_dir);
+            assert::eval_explicit_assert(assert, variables, http_response, cache, context_dir);
         asserts.push(assert_result);
     }
     asserts
@@ -171,7 +176,11 @@ fn eval_implicit_body_asserts(
                         start: spec_body.space0.source_info.end,
                         end: spec_body.space0.source_info.end,
                     };
-                    Err(Error::new(source_info, e.into(), true))
+                    Err(RunnerError::new(
+                        source_info,
+                        RunnerErrorKind::Http(e),
+                        true,
+                    ))
                 }
             };
             AssertResult::Body {
@@ -189,7 +198,11 @@ fn eval_implicit_body_asserts(
                         start: spec_body.space0.source_info.end,
                         end: spec_body.space0.source_info.end,
                     };
-                    Err(Error::new(source_info, e.into(), true))
+                    Err(RunnerError::new(
+                        source_info,
+                        RunnerErrorKind::Http(e),
+                        true,
+                    ))
                 }
             };
             AssertResult::Body {
@@ -210,7 +223,11 @@ fn eval_implicit_body_asserts(
                         start: spec_body.space0.source_info.end,
                         end: spec_body.space0.source_info.end,
                     };
-                    Err(Error::new(source_info, e.into(), true))
+                    Err(RunnerError::new(
+                        source_info,
+                        RunnerErrorKind::Http(e),
+                        true,
+                    ))
                 }
             };
             AssertResult::Body {
@@ -231,7 +248,11 @@ fn eval_implicit_body_asserts(
                         start: spec_body.space0.source_info.end,
                         end: spec_body.space0.source_info.end,
                     };
-                    Err(Error::new(source_info, e.into(), true))
+                    Err(RunnerError::new(
+                        source_info,
+                        RunnerErrorKind::Http(e),
+                        true,
+                    ))
                 }
             };
             AssertResult::Body {
@@ -254,7 +275,11 @@ fn eval_implicit_body_asserts(
                         start: spec_body.space0.source_info.end,
                         end: spec_body.space0.source_info.end,
                     };
-                    Err(Error::new(source_info, e.into(), true))
+                    Err(RunnerError::new(
+                        source_info,
+                        RunnerErrorKind::Http(e),
+                        true,
+                    ))
                 }
             };
             AssertResult::Body {
@@ -280,7 +305,11 @@ fn eval_implicit_body_asserts(
                         start: spec_body.space0.source_info.end,
                         end: spec_body.space0.source_info.end,
                     };
-                    Err(Error::new(source_info, e.into(), true))
+                    Err(RunnerError::new(
+                        source_info,
+                        RunnerErrorKind::Http(e),
+                        true,
+                    ))
                 }
             };
             AssertResult::Body {
@@ -304,7 +333,11 @@ fn eval_implicit_body_asserts(
                         start: spec_body.space0.source_info.end,
                         end: spec_body.space0.source_info.end,
                     };
-                    Err(Error::new(source_info, e.into(), true))
+                    Err(RunnerError::new(
+                        source_info,
+                        RunnerErrorKind::Http(e),
+                        true,
+                    ))
                 }
             };
             AssertResult::Body {
@@ -320,11 +353,12 @@ fn eval_implicit_body_asserts(
 pub fn eval_captures(
     response: &Response,
     http_response: &http::Response,
+    cache: &mut BodyCache,
     variables: &mut HashMap<String, Value>,
-) -> Result<Vec<CaptureResult>, Error> {
+) -> Result<Vec<CaptureResult>, RunnerError> {
     let mut captures = vec![];
-    for capture in &response.captures() {
-        let capture_result = capture::eval_capture(capture, variables, http_response)?;
+    for capture in response.captures() {
+        let capture_result = capture::eval_capture(capture, variables, http_response, cache)?;
         // Update variables now so the captures set is ready in case
         // the next captures reference this new variable.
         variables.insert(capture_result.name.clone(), capture_result.value.clone());
@@ -335,6 +369,8 @@ pub fn eval_captures(
 
 #[cfg(test)]
 mod tests {
+    use hurl_core::reader::Pos;
+
     use self::super::super::{assert, capture};
     use super::*;
     use crate::runner::Number;
@@ -388,20 +424,23 @@ mod tests {
     #[test]
     pub fn test_eval_asserts() {
         let variables = HashMap::new();
+        let mut cache = BodyCache::new();
+
         let context_dir = ContextDir::default();
         assert_eq!(
             eval_asserts(
                 &user_response(),
                 &variables,
                 &http::xml_two_users_http_response(),
+                &mut cache,
                 &context_dir,
             ),
             vec![AssertResult::Explicit {
                 actual: Ok(Some(Value::Number(Number::Integer(2)))),
                 source_info: SourceInfo::new(Pos::new(1, 22), Pos::new(1, 24)),
-                predicate_result: Some(Err(Error::new(
+                predicate_result: Some(Err(RunnerError::new(
                     SourceInfo::new(Pos::new(1, 0), Pos::new(1, 0)),
-                    RunnerError::AssertFailure {
+                    RunnerErrorKind::AssertFailure {
                         actual: "int <2>".to_string(),
                         expected: "int <3>".to_string(),
                         type_mismatch: false,
@@ -434,10 +473,13 @@ mod tests {
     #[test]
     pub fn test_eval_captures() {
         let mut variables = HashMap::new();
+        let mut cache = BodyCache::new();
+
         assert_eq!(
             eval_captures(
                 &user_response(),
                 &http::xml_two_users_http_response(),
+                &mut cache,
                 &mut variables,
             )
             .unwrap(),
