@@ -15,12 +15,16 @@
  * limitations under the License.
  *
  */
-use hurl_core::ast::{Pos, SourceInfo};
+use std::time::Duration;
+
+use hurl_core::ast::SourceInfo;
+use hurl_core::reader::Pos;
 
 use crate::http::{Call, Cookie};
-use crate::runner::error::Error;
+use crate::runner::error::RunnerError;
 use crate::runner::output::Output;
 use crate::runner::value::Value;
+use crate::runner::RunnerErrorKind;
 use crate::util::path::ContextDir;
 use crate::util::term::Stdout;
 
@@ -29,8 +33,8 @@ use crate::util::term::Stdout;
 pub struct HurlResult {
     /// The entries result for this run.
     pub entries: Vec<EntryResult>,
-    /// Duration in milliseconds of the run.
-    pub time_in_ms: u128,
+    /// Total duration of the run, including asserts and results computation.
+    pub duration: Duration,
     /// `true` if the run is successful, `false` if there has been runtime or asserts errors.
     pub success: bool,
     /// The list of cookies at the end of the run.
@@ -45,7 +49,7 @@ impl HurlResult {
     ///
     /// The errors are only the "effective" ones: those that are due to retry are
     /// ignored.
-    pub fn errors(&self) -> Vec<(&Error, SourceInfo)> {
+    pub fn errors(&self) -> Vec<(&RunnerError, SourceInfo)> {
         let mut errors = vec![];
         let mut next_entries = self.entries.iter().skip(1);
         for entry in self.entries.iter() {
@@ -81,8 +85,10 @@ pub struct EntryResult {
     /// List of asserts.
     pub asserts: Vec<AssertResult>,
     /// List of errors.
-    pub errors: Vec<Error>,
-    pub time_in_ms: u128,
+    pub errors: Vec<RunnerError>,
+
+    /// Effective duration of all the HTTP transfers, excluding asserts and captures processing.
+    pub transfer_duration: Duration,
     /// The entry has been executed with `--compressed` option:
     /// server is requested to send compressed response, and the response should be uncompressed
     /// when outputted on stdout.
@@ -98,7 +104,7 @@ impl Default for EntryResult {
             captures: vec![],
             asserts: vec![],
             errors: vec![],
-            time_in_ms: 0,
+            transfer_duration: Duration::from_millis(0),
             compressed: false,
         }
     }
@@ -140,19 +146,19 @@ pub enum AssertResult {
     },
     /// Implicit HTTP response header assert.
     Header {
-        actual: Result<String, Error>,
+        actual: Result<String, RunnerError>,
         expected: String,
         source_info: SourceInfo,
     },
     /// Implicit HTTP response body assert.
     Body {
-        actual: Result<Value, Error>,
-        expected: Result<Value, Error>,
+        actual: Result<Value, RunnerError>,
+        expected: Result<Value, RunnerError>,
         source_info: SourceInfo,
     },
     /// Explicit assert on HTTP response.
     Explicit {
-        actual: Result<Option<Value>, Error>,
+        actual: Result<Option<Value>, RunnerError>,
         source_info: SourceInfo,
         predicate_result: Option<PredicateResult>,
     },
@@ -170,7 +176,7 @@ pub struct CaptureResult {
     pub value: Value,
 }
 
-pub type PredicateResult = Result<(), Error>;
+pub type PredicateResult = Result<(), RunnerError>;
 
 impl EntryResult {
     /// Writes the last HTTP response of this entry result to this `output`.
@@ -182,7 +188,7 @@ impl EntryResult {
         context_dir: &ContextDir,
         stdout: &mut Stdout,
         source_info: SourceInfo,
-    ) -> Result<(), Error> {
+    ) -> Result<(), RunnerError> {
         let Some(call) = self.calls.last() else {
             return Ok(());
         };
@@ -191,12 +197,16 @@ impl EntryResult {
             let bytes = match response.uncompress_body() {
                 Ok(bytes) => bytes,
                 Err(e) => {
-                    return Err(Error::new(source_info, e.into(), false));
+                    return Err(RunnerError::new(
+                        source_info,
+                        RunnerErrorKind::Http(e),
+                        false,
+                    ));
                 }
             };
-            output.write(&bytes, stdout, Some(context_dir))
+            output.write_with_context_dir(&bytes, stdout, context_dir, source_info)
         } else {
-            output.write(&response.body, stdout, Some(context_dir))
+            output.write_with_context_dir(&response.body, stdout, context_dir, source_info)
         }
     }
 }
