@@ -18,6 +18,7 @@
 use std::fmt::Display;
 
 use crate::ast::*;
+use crate::typing::Count;
 
 /// Returns an HTML string of the Hurl file `hurl_file`.
 ///
@@ -157,7 +158,7 @@ impl HtmlFormatter {
     }
 
     fn fmt_status(&mut self, status: &Status) {
-        self.fmt_number(&status.value.to_string());
+        self.fmt_number(status.value.to_string());
     }
 
     fn fmt_section(&mut self, section: &Section) {
@@ -220,7 +221,7 @@ impl HtmlFormatter {
             OptionKind::ClientKey(filename) => self.fmt_filename(filename),
             OptionKind::Compressed(value) => self.fmt_bool_option(value),
             OptionKind::ConnectTo(value) => self.fmt_template(value),
-            OptionKind::Delay(value) => self.fmt_natural_option(value),
+            OptionKind::Delay(value) => self.fmt_duration_option(value),
             OptionKind::FollowLocation(value) => self.fmt_bool_option(value),
             OptionKind::FollowLocationTrusted(value) => self.fmt_bool_option(value),
             OptionKind::Http10(value) => self.fmt_bool_option(value),
@@ -230,16 +231,17 @@ impl HtmlFormatter {
             OptionKind::Insecure(value) => self.fmt_bool_option(value),
             OptionKind::IpV4(value) => self.fmt_bool_option(value),
             OptionKind::IpV6(value) => self.fmt_bool_option(value),
-            OptionKind::MaxRedirect(value) => self.fmt_natural_option(value),
+            OptionKind::MaxRedirect(value) => self.fmt_count_option(value),
             OptionKind::NetRc(value) => self.fmt_bool_option(value),
             OptionKind::NetRcFile(filename) => self.fmt_filename(filename),
             OptionKind::NetRcOptional(value) => self.fmt_bool_option(value),
             OptionKind::Output(filename) => self.fmt_filename(filename),
             OptionKind::PathAsIs(value) => self.fmt_bool_option(value),
             OptionKind::Proxy(value) => self.fmt_template(value),
+            OptionKind::Repeat(value) => self.fmt_count_option(value),
             OptionKind::Resolve(value) => self.fmt_template(value),
-            OptionKind::Retry(value) => self.fmt_retry_option(value),
-            OptionKind::RetryInterval(value) => self.fmt_natural_option(value),
+            OptionKind::Retry(value) => self.fmt_count_option(value),
+            OptionKind::RetryInterval(value) => self.fmt_duration_option(value),
             OptionKind::Skip(value) => self.fmt_bool_option(value),
             OptionKind::UnixSocket(value) => self.fmt_template(value),
             OptionKind::User(value) => self.fmt_template(value),
@@ -251,18 +253,17 @@ impl HtmlFormatter {
         self.fmt_lt(&option.line_terminator0);
     }
 
-    fn fmt_retry_option(&mut self, retry_option: &RetryOption) {
-        match retry_option {
-            RetryOption::Literal(retry) => self.fmt_retry(retry),
-            RetryOption::Expression(expr) => self.fmt_expr(expr),
-        };
+    fn fmt_count_option(&mut self, count_option: &CountOption) {
+        match count_option {
+            CountOption::Literal(repeat) => self.fmt_count(*repeat),
+            CountOption::Expression(expr) => self.fmt_expr(expr),
+        }
     }
 
-    fn fmt_retry(&mut self, retry: &Retry) {
-        match retry {
-            Retry::Finite(n) => self.fmt_number(n),
-            Retry::Infinite => self.fmt_number(-1),
-            Retry::None => self.fmt_number(0),
+    fn fmt_count(&mut self, count: Count) {
+        match count {
+            Count::Finite(n) => self.fmt_number(n),
+            Count::Infinite => self.fmt_number(-1),
         };
     }
 
@@ -553,18 +554,6 @@ impl HtmlFormatter {
     }
 
     fn fmt_multiline_string(&mut self, multiline_string: &MultilineString, as_body: bool) {
-        if let MultilineString::OneLineText(line) = multiline_string {
-            let line = format!("```{line}```");
-            if as_body {
-                self.fmt_span_open("multiline");
-                self.fmt_span("line", &line);
-                self.fmt_span_close();
-            } else {
-                self.fmt_span("multiline", &line);
-            }
-            return;
-        }
-
         // The multiline spans multiple newlines. We distinguish cases for multiline
         // as a body and multiline as a predicate value. When used as a body, we can embed
         // span lines with the multiline span. Used as a predicate, we have to break the multiline
@@ -621,7 +610,16 @@ impl HtmlFormatter {
         // ```
         let lang = multiline_string.lang();
         if as_body {
-            let body = format!("```{lang}\n{multiline_string}```");
+            let mut attributes = String::new();
+            for (i, attribute) in multiline_string.attributes.iter().enumerate() {
+                if i > 0 || !lang.is_empty() {
+                    attributes.push_str(", ");
+                }
+                attributes.push_str(attribute.to_string().as_str());
+            }
+            // Keep original encoded string
+            let multiline_string_encoded = multiline_string.kind.to_encoded_string();
+            let body = format!("```{lang}{attributes}\n{multiline_string_encoded}```");
             let body = format_multilines(&body);
             self.fmt_span("multiline", &body);
         } else {
@@ -689,10 +687,15 @@ impl HtmlFormatter {
         }
     }
 
-    fn fmt_natural_option(&mut self, value: &NaturalOption) {
+    fn fmt_duration_option(&mut self, value: &DurationOption) {
         match value {
-            NaturalOption::Literal(value) => self.fmt_span("number", &value.to_string()),
-            NaturalOption::Expression(value) => self.fmt_expr(value),
+            DurationOption::Literal(value) => {
+                self.fmt_span("number", &value.value.to_string());
+                if let Some(unit) = value.unit {
+                    self.fmt_span("unit", &unit.to_string());
+                }
+            }
+            DurationOption::Expression(value) => self.fmt_expr(value),
         }
     }
 
@@ -828,6 +831,7 @@ impl HtmlFormatter {
                 self.fmt_space(space0);
                 self.fmt_template(fmt);
             }
+            FilterValue::ToFloat => self.fmt_span("filter-type", "toFloat"),
             FilterValue::ToInt => self.fmt_span("filter-type", "toInt"),
             FilterValue::UrlDecode => self.fmt_span("filter-type", "urlDecode"),
             FilterValue::UrlEncode => self.fmt_span("filter-type", "urlEncode"),
@@ -877,6 +881,37 @@ impl Template {
     }
 }
 
+impl MultilineStringKind {
+    fn to_encoded_string(&self) -> String {
+        match self {
+            MultilineStringKind::Text(text)
+            | MultilineStringKind::Json(text)
+            | MultilineStringKind::Xml(text) => text.value.to_encoded_string(),
+            MultilineStringKind::GraphQl(graphql) => graphql.to_encoded_string(),
+        }
+    }
+}
+
+impl GraphQl {
+    fn to_encoded_string(&self) -> String {
+        let mut s = self.value.to_encoded_string();
+        if let Some(vars) = &self.variables {
+            s.push_str(&vars.to_encoded_string());
+        }
+        s
+    }
+}
+
+impl GraphQlVariables {
+    fn to_encoded_string(&self) -> String {
+        let mut s = "variables".to_string();
+        s.push_str(&self.space.value);
+        s.push_str(&self.value.encoded());
+        s.push_str(&self.whitespace.value);
+        s
+    }
+}
+
 fn encode_html(s: String) -> String {
     s.replace('>', "&gt;").replace('<', "&lt;")
 }
@@ -900,55 +935,15 @@ fn pop_str(string: &mut String, suffix: &str) {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::reader::Pos;
 
     #[test]
     fn test_multiline_string() {
-        // ``````
-        let multiline_string = MultilineString::OneLineText(Template {
-            delimiter: None,
-            elements: vec![TemplateElement::String {
-                value: String::new(),
-                encoded: "unused".to_string(),
-            }],
-            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-        });
-        let mut fmt = HtmlFormatter::new();
-        fmt.fmt_multiline_string(&multiline_string, false);
-        assert_eq!(fmt.buffer, "<span class=\"multiline\">``````</span>");
-        let mut fmt = HtmlFormatter::new();
-        fmt.fmt_multiline_string(&multiline_string, true);
-        assert_eq!(
-            fmt.buffer,
-            "<span class=\"multiline\">\
-                <span class=\"line\">``````</span>\
-            </span>"
-        );
-
-        // ```hello```
-        let multiline_string = MultilineString::OneLineText(Template {
-            delimiter: None,
-            elements: vec![TemplateElement::String {
-                value: "hello".to_string(),
-                encoded: "unused".to_string(),
-            }],
-            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-        });
-        let mut fmt = HtmlFormatter::new();
-        fmt.fmt_multiline_string(&multiline_string, false);
-        assert_eq!(fmt.buffer, "<span class=\"multiline\">```hello```</span>");
-        let mut fmt = HtmlFormatter::new();
-        fmt.fmt_multiline_string(&multiline_string, true);
-        assert_eq!(
-            fmt.buffer,
-            "<span class=\"multiline\">\
-                <span class=\"line\">```hello```</span>\
-            </span>"
-        );
         // ```
         // line1
         // line2
         // ```
-        let multiline_string = MultilineString::Text(Text {
+        let kind = MultilineStringKind::Text(Text {
             space: Whitespace {
                 value: String::new(),
                 source_info: SourceInfo {
@@ -967,11 +962,13 @@ mod tests {
                 delimiter: None,
                 elements: vec![TemplateElement::String {
                     value: "line1\nline2\n".to_string(),
-                    encoded: "unused".to_string(),
+                    encoded: "line1\nline2\n".to_string(),
                 }],
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
         });
+        let attributes = vec![];
+        let multiline_string = MultilineString { kind, attributes };
         let mut fmt = HtmlFormatter::new();
         fmt.fmt_multiline_string(&multiline_string, true);
         assert_eq!(
