@@ -15,27 +15,26 @@
  * limitations under the License.
  *
  */
-pub type ParseResult<T> = Result<T, Error>;
-pub type ParseFunc<T> = fn(&mut Reader) -> ParseResult<T>;
+pub type ParseResult<T> = Result<T, ParseError>;
 
 pub fn parse_hurl_file(s: &str) -> ParseResult<HurlFile> {
     let mut reader = Reader::new(s);
     parsers::hurl_file(&mut reader)
 }
 
-pub use self::error::{Error, JsonErrorVariant, ParseError};
+pub use self::error::{JsonErrorVariant, ParseError, ParseErrorKind};
 pub use self::json::{
     boolean_value as parse_json_boolean, null_value as parse_json_null,
     number_value as parse_json_number, parse as parse_json,
 };
-pub use self::reader::Reader;
 pub use self::template::templatize;
 use crate::ast::HurlFile;
+use crate::reader::Reader;
 
 mod base64;
 mod bytes;
-mod combinators;
 mod cookiepath;
+mod duration;
 mod error;
 mod expr;
 mod filename;
@@ -51,7 +50,6 @@ mod predicate;
 mod predicate_value;
 mod primitives;
 mod query;
-mod reader;
 mod sections;
 mod string;
 mod template;
