@@ -16,24 +16,26 @@
  *
  */
 use crate::ast::*;
-use crate::parser::combinators::*;
+use crate::combinator::{choice, non_recover};
+use crate::parser::duration::duration;
 use crate::parser::error::*;
-use crate::parser::number::{integer, natural, number};
+use crate::parser::number::{integer, number};
 use crate::parser::primitives::*;
-use crate::parser::reader::Reader;
 use crate::parser::string::*;
 use crate::parser::{expr, filename, filename_password, ParseResult};
+use crate::reader::Reader;
+use crate::typing::Count;
 
 /// Parse an option in an `[Options]` section.
 pub fn parse(reader: &mut Reader) -> ParseResult<EntryOption> {
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
-    let start = reader.state.pos;
+    let start = reader.cursor();
     // We accept '_' even if there is no option name with this character. We do this to be able to
     // enter the parsing of the option name and to have better error description (ex: 'max-redirs'
     // vs 'max_redirs').
     let option =
-        reader.read_while(|c| c.is_ascii_alphanumeric() || *c == '-' || *c == '.' || *c == '_');
+        reader.read_while(|c| c.is_ascii_alphanumeric() || c == '-' || c == '.' || c == '_');
     let space1 = zero_or_more_spaces(reader)?;
     try_literal(":", reader)?;
     let space2 = zero_or_more_spaces(reader)?;
@@ -61,6 +63,7 @@ pub fn parse(reader: &mut Reader) -> ParseResult<EntryOption> {
         "output" => option_output(reader)?,
         "path-as-is" => option_path_as_is(reader)?,
         "proxy" => option_proxy(reader)?,
+        "repeat" => option_repeat(reader)?,
         "resolve" => option_resolve(reader)?,
         "retry" => option_retry(reader)?,
         "retry-interval" => option_retry_interval(reader)?,
@@ -71,10 +74,10 @@ pub fn parse(reader: &mut Reader) -> ParseResult<EntryOption> {
         "verbose" => option_verbose(reader)?,
         "very-verbose" => option_very_verbose(reader)?,
         _ => {
-            return Err(Error::new(
-                start,
+            return Err(ParseError::new(
+                start.pos,
                 false,
-                ParseError::InvalidOption(option.to_string()),
+                ParseErrorKind::InvalidOption(option.to_string()),
             ))
         }
     };
@@ -106,7 +109,7 @@ fn option_cert(reader: &mut Reader) -> ParseResult<OptionKind> {
 }
 
 fn option_compressed(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::Compressed(value))
 }
 
@@ -116,52 +119,52 @@ fn option_connect_to(reader: &mut Reader) -> ParseResult<OptionKind> {
 }
 
 fn option_delay(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = natural_option(reader)?;
+    let value = duration_option(reader)?;
     Ok(OptionKind::Delay(value))
 }
 
 fn option_follow_location(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::FollowLocation(value))
 }
 
 fn option_follow_location_trusted(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::FollowLocationTrusted(value))
 }
 
 fn option_http_10(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::Http10(value))
 }
 
 fn option_http_11(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::Http11(value))
 }
 
 fn option_http_2(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::Http2(value))
 }
 
 fn option_http_3(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::Http3(value))
 }
 
 fn option_insecure(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::Insecure(value))
 }
 
 fn option_ipv4(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::IpV4(value))
 }
 
 fn option_ipv6(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::IpV6(value))
 }
 
@@ -171,12 +174,12 @@ fn option_key(reader: &mut Reader) -> ParseResult<OptionKind> {
 }
 
 fn option_max_redirect(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(natural_option, reader)?;
+    let value = non_recover(count_option, reader)?;
     Ok(OptionKind::MaxRedirect(value))
 }
 
 fn option_netrc(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::NetRc(value))
 }
 
@@ -186,7 +189,7 @@ fn option_netrc_file(reader: &mut Reader) -> ParseResult<OptionKind> {
 }
 
 fn option_netrc_optional(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::NetRcOptional(value))
 }
 
@@ -196,7 +199,7 @@ fn option_output(reader: &mut Reader) -> ParseResult<OptionKind> {
 }
 
 fn option_path_as_is(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::PathAsIs(value))
 }
 
@@ -205,23 +208,28 @@ fn option_proxy(reader: &mut Reader) -> ParseResult<OptionKind> {
     Ok(OptionKind::Proxy(value))
 }
 
+fn option_repeat(reader: &mut Reader) -> ParseResult<OptionKind> {
+    let value = non_recover(count_option, reader)?;
+    Ok(OptionKind::Repeat(value))
+}
+
 fn option_resolve(reader: &mut Reader) -> ParseResult<OptionKind> {
     let value = unquoted_template(reader)?;
     Ok(OptionKind::Resolve(value))
 }
 
 fn option_retry(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = retry_option(reader)?;
+    let value = non_recover(count_option, reader)?;
     Ok(OptionKind::Retry(value))
 }
 
 fn option_retry_interval(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(natural_option, reader)?;
+    let value = non_recover(duration_option, reader)?;
     Ok(OptionKind::RetryInterval(value))
 }
 
 fn option_skip(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::Skip(value))
 }
 
@@ -241,79 +249,81 @@ fn option_variable(reader: &mut Reader) -> ParseResult<OptionKind> {
 }
 
 fn option_verbose(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::Verbose(value))
 }
 
 fn option_very_verbose(reader: &mut Reader) -> ParseResult<OptionKind> {
-    let value = nonrecover(boolean_option, reader)?;
+    let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::VeryVerbose(value))
 }
 
-fn retry(reader: &mut Reader) -> ParseResult<Retry> {
-    let pos = reader.state.pos;
-    let value = nonrecover(integer, reader)?;
+fn count(reader: &mut Reader) -> ParseResult<Count> {
+    let start = reader.cursor();
+    let value = non_recover(integer, reader)?;
     if value == -1 {
-        Ok(Retry::Infinite)
-    } else if value == 0 {
-        Ok(Retry::None)
-    } else if value > 0 {
-        Ok(Retry::Finite(value as usize))
+        Ok(Count::Infinite)
+    } else if value >= 0 {
+        Ok(Count::Finite(value as usize))
     } else {
-        let inner = ParseError::Expecting {
-            value: "Expecting a retry value".to_string(),
+        let kind = ParseErrorKind::Expecting {
+            value: "Expecting a count value".to_string(),
         };
-        Err(Error::new(pos, false, inner))
+        Err(ParseError::new(start.pos, false, kind))
     }
 }
 
 fn boolean_option(reader: &mut Reader) -> ParseResult<BooleanOption> {
-    let start = reader.state;
+    let start = reader.cursor();
     match boolean(reader) {
         Ok(v) => Ok(BooleanOption::Literal(v)),
         Err(_) => {
-            reader.state = start;
+            reader.seek(start);
             let exp = expr::parse(reader).map_err(|e| {
-                let inner = ParseError::Expecting {
+                let kind = ParseErrorKind::Expecting {
                     value: "true|false".to_string(),
                 };
-                Error::new(e.pos, false, inner)
+                ParseError::new(e.pos, false, kind)
             })?;
             Ok(BooleanOption::Expression(exp))
         }
     }
 }
 
-fn natural_option(reader: &mut Reader) -> ParseResult<NaturalOption> {
-    let start = reader.state;
-    match natural(reader) {
-        Ok(v) => Ok(NaturalOption::Literal(v)),
+fn count_option(reader: &mut Reader) -> ParseResult<CountOption> {
+    let start = reader.cursor();
+    match count(reader) {
+        Ok(v) => Ok(CountOption::Literal(v)),
         Err(_) => {
-            reader.state = start;
+            reader.seek(start);
             let exp = expr::parse(reader).map_err(|e| {
-                let inner = ParseError::Expecting {
-                    value: "integer".to_string(),
+                let kind = ParseErrorKind::Expecting {
+                    value: "integer >= -1".to_string(),
                 };
-                Error::new(e.pos, false, inner)
+                ParseError::new(e.pos, false, kind)
             })?;
-            Ok(NaturalOption::Expression(exp))
+            Ok(CountOption::Expression(exp))
         }
     }
 }
 
-fn retry_option(reader: &mut Reader) -> ParseResult<RetryOption> {
-    let start = reader.state;
-    match retry(reader) {
-        Ok(v) => Ok(RetryOption::Literal(v)),
-        Err(_) => {
-            reader.state = start;
-            let exp = expr::parse(reader).map_err(|e| {
-                let inner = ParseError::Expecting {
-                    value: "integer".to_string(),
-                };
-                Error::new(e.pos, false, inner)
-            })?;
-            Ok(RetryOption::Expression(exp))
+fn duration_option(reader: &mut Reader) -> ParseResult<DurationOption> {
+    let start = reader.cursor();
+    match duration(reader) {
+        Ok(v) => Ok(DurationOption::Literal(v)),
+        Err(e) => {
+            if e.recoverable {
+                reader.seek(start);
+                let exp = expr::parse(reader).map_err(|e| {
+                    let kind = ParseErrorKind::Expecting {
+                        value: "integer".to_string(),
+                    };
+                    ParseError::new(e.pos, false, kind)
+                })?;
+                Ok(DurationOption::Expression(exp))
+            } else {
+                Err(e)
+            }
         }
     }
 }
@@ -333,13 +343,13 @@ fn variable_definition(reader: &mut Reader) -> ParseResult<VariableDefinition> {
 }
 
 fn variable_name(reader: &mut Reader) -> ParseResult<String> {
-    let start = reader.state;
-    let name = reader.read_while(|c| c.is_alphanumeric() || *c == '_' || *c == '-');
+    let start = reader.cursor();
+    let name = reader.read_while(|c| c.is_alphanumeric() || c == '_' || c == '-');
     if name.is_empty() {
-        let inner = ParseError::Expecting {
+        let kind = ParseErrorKind::Expecting {
             value: "variable name".to_string(),
         };
-        return Err(Error::new(start.pos, false, inner));
+        return Err(ParseError::new(start.pos, false, kind));
     }
     Ok(name)
 }
@@ -371,17 +381,17 @@ fn variable_value(reader: &mut Reader) -> ParseResult<VariableValue> {
         reader,
     )
     .map_err(|e| {
-        let inner = ParseError::Expecting {
+        let kind = ParseErrorKind::Expecting {
             value: "variable value".to_string(),
         };
-        Error::new(e.pos, false, inner)
+        ParseError::new(e.pos, false, kind)
     })
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::ast::Pos;
+    use crate::reader::Pos;
 
     #[test]
     fn test_option_insecure() {
@@ -549,9 +559,9 @@ mod tests {
         assert!(!error.recoverable);
         assert_eq!(error.pos, Pos { line: 1, column: 8 });
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
-                value: "integer".to_string()
+            error.kind,
+            ParseErrorKind::Expecting {
+                value: "integer >= -1".to_string()
             }
         );
     }
