@@ -17,15 +17,15 @@
  */
 use crate::ast::VersionValue::VersionAny;
 use crate::ast::*;
+use crate::combinator::{optional, zero_or_more};
 use crate::parser::bytes::*;
-use crate::parser::combinators::*;
 use crate::parser::error::*;
 use crate::parser::number::natural;
 use crate::parser::primitives::*;
-use crate::parser::reader::Reader;
 use crate::parser::sections::*;
 use crate::parser::url::url;
 use crate::parser::ParseResult;
+use crate::reader::Reader;
 
 pub fn hurl_file(reader: &mut Reader) -> ParseResult<HurlFile> {
     let entries = zero_or_more(entry, reader)?;
@@ -47,7 +47,7 @@ fn entry(reader: &mut Reader) -> ParseResult<Entry> {
 }
 
 fn request(reader: &mut Reader) -> ParseResult<Request> {
-    let start = reader.state;
+    let start = reader.cursor();
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
     let m = method(reader)?;
@@ -58,16 +58,16 @@ fn request(reader: &mut Reader) -> ParseResult<Request> {
     let headers = zero_or_more(key_value, reader)?;
     let sections = request_sections(reader)?;
     let b = optional(body, reader)?;
-    let source_info = SourceInfo::new(start.pos, reader.state.pos);
+    let source_info = SourceInfo::new(start.pos, reader.cursor().pos);
 
     // Check duplicated section
     let mut section_names = vec![];
     for section in sections.clone() {
         if section_names.contains(&section.name().to_string()) {
-            return Err(Error::new(
+            return Err(ParseError::new(
                 section.source_info.start,
                 false,
-                ParseError::DuplicateSection,
+                ParseErrorKind::DuplicateSection,
             ));
         } else {
             section_names.push(section.name().to_string());
@@ -89,7 +89,7 @@ fn request(reader: &mut Reader) -> ParseResult<Request> {
 }
 
 fn response(reader: &mut Reader) -> ParseResult<Response> {
-    let start = reader.state;
+    let start = reader.cursor();
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
     let _version = version(reader)?;
@@ -109,29 +109,29 @@ fn response(reader: &mut Reader) -> ParseResult<Response> {
         headers,
         sections,
         body: b,
-        source_info: SourceInfo::new(start.pos, reader.state.pos),
+        source_info: SourceInfo::new(start.pos, reader.cursor().pos),
     })
 }
 
 fn method(reader: &mut Reader) -> ParseResult<Method> {
     if reader.is_eof() {
-        let inner = ParseError::Method {
+        let kind = ParseErrorKind::Method {
             name: "<EOF>".to_string(),
         };
-        return Err(Error::new(reader.state.pos, true, inner));
+        return Err(ParseError::new(reader.cursor().pos, true, kind));
     }
-    let start = reader.state;
+    let start = reader.cursor();
     let name = reader.read_while(|c| c.is_ascii_alphabetic());
     if name.is_empty() || name.to_uppercase() != name {
-        let inner = ParseError::Method { name };
-        Err(Error::new(start.pos, false, inner))
+        let kind = ParseErrorKind::Method { name };
+        Err(ParseError::new(start.pos, false, kind))
     } else {
         Ok(Method(name))
     }
 }
 
 fn version(reader: &mut Reader) -> ParseResult<Version> {
-    let start = reader.state;
+    let start = reader.cursor();
     try_literal("HTTP", reader)?;
 
     let next_c = reader.peek();
@@ -148,33 +148,33 @@ fn version(reader: &mut Reader) -> ParseResult<Version> {
                 if try_literal(s, reader).is_ok() {
                     return Ok(Version {
                         value: value.clone(),
-                        source_info: SourceInfo::new(start.pos, reader.state.pos),
+                        source_info: SourceInfo::new(start.pos, reader.cursor().pos),
                     });
                 }
             }
-            Err(Error::new(start.pos, false, ParseError::Version))
+            Err(ParseError::new(start.pos, false, ParseErrorKind::Version))
         }
         Some(' ') | Some('\t') => Ok(Version {
             value: VersionAny,
-            source_info: SourceInfo::new(start.pos, reader.state.pos),
+            source_info: SourceInfo::new(start.pos, reader.cursor().pos),
         }),
-        _ => Err(Error::new(start.pos, false, ParseError::Version)),
+        _ => Err(ParseError::new(start.pos, false, ParseErrorKind::Version)),
     }
 }
 
 fn status(reader: &mut Reader) -> ParseResult<Status> {
-    let start = reader.state.pos;
+    let start = reader.cursor();
     let value = match try_literal("*", reader) {
         Ok(_) => StatusValue::Any,
         Err(_) => match natural(reader) {
             Ok(value) => StatusValue::Specific(value),
-            Err(_) => return Err(Error::new(start, false, ParseError::Status)),
+            Err(_) => return Err(ParseError::new(start.pos, false, ParseErrorKind::Status)),
         },
     };
-    let end = reader.state.pos;
+    let end = reader.cursor();
     Ok(Status {
         value,
-        source_info: SourceInfo { start, end },
+        source_info: SourceInfo::new(start.pos, end.pos),
     })
 }
 
@@ -195,6 +195,7 @@ fn body(reader: &mut Reader) -> ParseResult<Body> {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::reader::Pos;
 
     #[test]
     fn test_hurl_file() {
@@ -208,7 +209,7 @@ mod tests {
         let mut reader = Reader::new("GET http://google.fr");
         let e = entry(&mut reader).unwrap();
         assert_eq!(e.request.method, Method("GET".to_string()));
-        assert_eq!(reader.state.cursor, 20);
+        assert_eq!(reader.cursor().index, 20);
     }
 
     #[test]
@@ -217,26 +218,26 @@ mod tests {
 
         let e = entry(&mut reader).unwrap();
         assert_eq!(e.request.method, Method("GET".to_string()));
-        assert_eq!(reader.state.cursor, 21);
-        assert_eq!(reader.state.pos.line, 2);
+        assert_eq!(reader.cursor().index, 21);
+        assert_eq!(reader.cursor().pos.line, 2);
 
         let e = entry(&mut reader).unwrap();
         assert_eq!(e.request.method, Method("GET".to_string()));
-        assert_eq!(reader.state.cursor, 41);
-        assert_eq!(reader.state.pos.line, 2);
+        assert_eq!(reader.cursor().index, 41);
+        assert_eq!(reader.cursor().pos.line, 2);
 
         let mut reader =
             Reader::new("GET http://google.fr # comment1\nGET http://google.fr # comment2");
 
         let e = entry(&mut reader).unwrap();
         assert_eq!(e.request.method, Method("GET".to_string()));
-        assert_eq!(reader.state.cursor, 32);
-        assert_eq!(reader.state.pos.line, 2);
+        assert_eq!(reader.cursor().index, 32);
+        assert_eq!(reader.cursor().pos.line, 2);
 
         let e = entry(&mut reader).unwrap();
         assert_eq!(e.request.method, Method("GET".to_string()));
-        assert_eq!(reader.state.cursor, 63);
-        assert_eq!(reader.state.pos.line, 2);
+        assert_eq!(reader.cursor().index, 63);
+        assert_eq!(reader.cursor().pos.line, 2);
     }
 
     #[test]
@@ -286,7 +287,7 @@ mod tests {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 21)),
         };
         assert_eq!(request(&mut reader).unwrap(), default_request);
-        assert_eq!(reader.state.cursor, 20);
+        assert_eq!(reader.cursor().index, 20);
 
         let mut reader = Reader::new("GET  http://google.fr # comment");
         let default_request = Request {
@@ -328,12 +329,12 @@ mod tests {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 32)),
         };
         assert_eq!(request(&mut reader).unwrap(), default_request);
-        assert_eq!(reader.state.cursor, 31);
+        assert_eq!(reader.cursor().index, 31);
 
         let mut reader = Reader::new("GET http://google.fr\nGET http://google.fr");
         let r = request(&mut reader).unwrap();
         assert_eq!(r.method, Method("GET".to_string()));
-        assert_eq!(reader.state.cursor, 21);
+        assert_eq!(reader.cursor().index, 21);
         let r = request(&mut reader).unwrap();
         assert_eq!(r.method, Method("GET".to_string()));
     }
@@ -354,24 +355,27 @@ mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 1)),
                 },
-                value: Bytes::MultilineString(MultilineString::Text(Text {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(2, 4)),
-                    },
-                    newline: Whitespace {
-                        source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(3, 1)),
-                        value: "\n".to_string(),
-                    },
-                    value: Template {
-                        elements: vec![TemplateElement::String {
-                            value: String::from("Hello World!\n"),
-                            encoded: String::from("Hello World!\n"),
-                        }],
-                        delimiter: None,
-                        source_info: SourceInfo::new(Pos::new(3, 1), Pos::new(4, 1)),
-                    },
-                })),
+                value: Bytes::MultilineString(MultilineString {
+                    kind: MultilineStringKind::Text(Text {
+                        space: Whitespace {
+                            value: String::new(),
+                            source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(2, 4)),
+                        },
+                        newline: Whitespace {
+                            source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(3, 1)),
+                            value: "\n".to_string(),
+                        },
+                        value: Template {
+                            elements: vec![TemplateElement::String {
+                                value: String::from("Hello World!\n"),
+                                encoded: String::from("Hello World!\n"),
+                            }],
+                            delimiter: None,
+                            source_info: SourceInfo::new(Pos::new(3, 1), Pos::new(4, 1)),
+                        },
+                    }),
+                    attributes: vec![]
+                }),
                 line_terminator0: LineTerminator {
                     space0: Whitespace {
                         value: String::new(),
@@ -462,20 +466,20 @@ mod tests {
         let mut reader = Reader::new("xxx ");
         let error = method(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(reader.state.cursor, 3);
+        assert_eq!(reader.cursor().index, 3);
 
         let mut reader = Reader::new("");
         let error = method(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(reader.state.cursor, 0);
+        assert_eq!(reader.cursor().index, 0);
 
         let mut reader = Reader::new("GET ");
         assert_eq!(method(&mut reader).unwrap(), Method("GET".to_string()));
-        assert_eq!(reader.state.cursor, 3);
+        assert_eq!(reader.cursor().index, 3);
 
         let mut reader = Reader::new("CUSTOM");
         assert_eq!(method(&mut reader).unwrap(), Method("CUSTOM".to_string()));
-        assert_eq!(reader.state.cursor, 6);
+        assert_eq!(reader.cursor().index, 6);
     }
 
     #[test]
@@ -485,14 +489,14 @@ mod tests {
             version(&mut reader).unwrap().value,
             VersionValue::VersionAny
         );
-        assert_eq!(reader.state.cursor, 4);
+        assert_eq!(reader.cursor().index, 4);
 
         let mut reader = Reader::new("HTTP\t200");
         assert_eq!(
             version(&mut reader).unwrap().value,
             VersionValue::VersionAny
         );
-        assert_eq!(reader.state.cursor, 4);
+        assert_eq!(reader.cursor().index, 4);
 
         let mut reader = Reader::new("HTTP/1.1 200");
         assert_eq!(version(&mut reader).unwrap().value, VersionValue::Version11);
@@ -545,7 +549,7 @@ mod tests {
                 ],
             })
         );
-        assert_eq!(reader.state.cursor, 8);
+        assert_eq!(reader.cursor().index, 8);
 
         let mut reader = Reader::new("{}");
         let b = body(&mut reader).unwrap();
@@ -557,7 +561,7 @@ mod tests {
                 elements: vec![],
             })
         );
-        assert_eq!(reader.state.cursor, 2);
+        assert_eq!(reader.cursor().index, 2);
 
         let mut reader = Reader::new("# comment\n {} # comment\nxxx");
         let b = body(&mut reader).unwrap();
@@ -569,7 +573,7 @@ mod tests {
                 elements: vec![],
             })
         );
-        assert_eq!(reader.state.cursor, 24);
+        assert_eq!(reader.cursor().index, 24);
 
         let mut reader = Reader::new("{x");
         let error = body(&mut reader).err().unwrap();
