@@ -16,13 +16,13 @@
  *
  */
 use crate::ast::*;
-use crate::parser::combinators::*;
+use crate::combinator::choice;
 use crate::parser::multiline::multiline_string;
 use crate::parser::number::number;
 use crate::parser::primitives::*;
-use crate::parser::reader::Reader;
 use crate::parser::string::*;
-use crate::parser::{expr, Error, ParseError, ParseResult};
+use crate::parser::{expr, ParseError, ParseErrorKind, ParseResult};
+use crate::reader::Reader;
 
 pub fn predicate_value(reader: &mut Reader) -> ParseResult<PredicateValue> {
     choice(
@@ -71,12 +71,12 @@ pub fn predicate_value(reader: &mut Reader) -> ParseResult<PredicateValue> {
         reader,
     )
     .map_err(|e| {
-        let inner = if e.recoverable {
-            ParseError::PredicateValue
+        let kind = if e.recoverable {
+            ParseErrorKind::PredicateValue
         } else {
-            e.inner
+            e.kind
         };
-        Error::new(e.pos, false, inner)
+        ParseError::new(e.pos, false, kind)
     })
 }
 
@@ -84,7 +84,8 @@ pub fn predicate_value(reader: &mut Reader) -> ParseResult<PredicateValue> {
 mod tests {
 
     use super::*;
-    use crate::parser::ParseError;
+    use crate::parser::ParseErrorKind;
+    use crate::reader::Pos;
 
     #[test]
     fn test_predicate_value() {
@@ -115,7 +116,7 @@ mod tests {
         let mut reader = Reader::new("xx");
         let error = predicate_value(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(error.inner, ParseError::PredicateValue);
+        assert_eq!(error.kind, ParseErrorKind::PredicateValue);
         assert!(!error.recoverable);
     }
 
@@ -131,8 +132,8 @@ mod tests {
             }
         );
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: "\"".to_string()
             }
         );
