@@ -15,10 +15,10 @@
  * limitations under the License.
  *
  */
-use crate::ast::{Expr, Pos, SourceInfo, TemplateElement};
+use crate::ast::{Expr, SourceInfo, TemplateElement};
 use crate::parser::primitives::{literal, try_literal};
-use crate::parser::reader::*;
 use crate::parser::{error, expr, ParseResult};
+use crate::reader::{Pos, Reader};
 
 pub struct EncodedString {
     pub source_info: SourceInfo,
@@ -90,11 +90,7 @@ pub fn templatize(encoded_string: EncodedString) -> ParseResult<Vec<TemplateElem
 
             State::FirstCloseBracket => {
                 if s.as_str() == "}" {
-                    let mut reader = Reader::new(encoded.as_str());
-                    reader.state = ReaderState {
-                        cursor: 0,
-                        pos: expression_start.unwrap(),
-                    };
+                    let mut reader = Reader::with_pos(encoded.as_str(), expression_start.unwrap());
                     let expression = expr::parse2(&mut reader)?;
                     elements.push(TemplateElement::Expression(expression));
                     value = String::new();
@@ -118,13 +114,13 @@ pub fn templatize(encoded_string: EncodedString) -> ParseResult<Vec<TemplateElem
             encoded.push('{');
         }
         State::Template | State::FirstCloseBracket => {
-            let inner = error::ParseError::Expecting {
+            let kind = error::ParseErrorKind::Expecting {
                 value: "}}".to_string(),
             };
-            return Err(error::Error::new(
+            return Err(error::ParseError::new(
                 encoded_string.source_info.end,
                 false,
-                inner,
+                kind,
             ));
         }
     }
@@ -301,8 +297,8 @@ mod tests {
         let error = templatize(encoded_string).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 4 });
         assert_eq!(
-            error.inner,
-            error::ParseError::Expecting {
+            error.kind,
+            error::ParseErrorKind::Expecting {
                 value: "}}".to_string()
             }
         );
