@@ -18,6 +18,7 @@
 use base64::engine::general_purpose;
 use base64::Engine;
 use hurl_core::ast::*;
+use hurl_core::typing::{Count, Duration};
 
 use super::serialize_json::*;
 
@@ -90,6 +91,18 @@ impl ToJson for Request {
         if let Some(body) = &self.body {
             attributes.push(("body".to_string(), body.to_json()));
         }
+
+        // Request comments (can be used to check custom commands)
+        let comments: Vec<_> = self
+            .line_terminators
+            .iter()
+            .filter_map(|l| l.comment.as_ref())
+            .collect();
+        if !comments.is_empty() {
+            let comments = comments.iter().map(|c| c.to_json()).collect();
+            attributes.push(("comments".to_string(), JValue::List(comments)));
+        }
+
         JValue::Object(attributes)
     }
 }
@@ -168,10 +181,22 @@ impl ToJson for Bytes {
                 // base64,SGVsbG8gd29ybGQ=;
                 // ~~~
                 let lang = match multi {
-                    MultilineString::OneLineText(_) | MultilineString::Text(_) => "text",
-                    MultilineString::Json(_) => "json",
-                    MultilineString::Xml(_) => "xml",
-                    MultilineString::GraphQl(_) => "graphql",
+                    MultilineString {
+                        kind: MultilineStringKind::Text(_),
+                        ..
+                    } => "text",
+                    MultilineString {
+                        kind: MultilineStringKind::Json(_),
+                        ..
+                    } => "json",
+                    MultilineString {
+                        kind: MultilineStringKind::Xml(_),
+                        ..
+                    } => "xml",
+                    MultilineString {
+                        kind: MultilineStringKind::GraphQl(_),
+                        ..
+                    } => "graphql",
                 };
                 JValue::Object(vec![
                     ("type".to_string(), JValue::String(lang.to_string())),
@@ -272,13 +297,6 @@ impl ToJson for Cookie {
 
 impl ToJson for EntryOption {
     fn to_json(&self) -> JValue {
-        let mut attributes = vec![];
-
-        let name = "name".to_string();
-        let value = JValue::String(self.kind.name().to_string());
-        attributes.push((name, value));
-
-        let name = "value".to_string();
         let value = match &self.kind {
             OptionKind::AwsSigV4(value) => JValue::String(value.to_string()),
             OptionKind::CaCertificate(filename) => JValue::String(filename.to_string()),
@@ -303,6 +321,7 @@ impl ToJson for EntryOption {
             OptionKind::Output(filename) => JValue::String(filename.to_string()),
             OptionKind::PathAsIs(value) => value.to_json(),
             OptionKind::Proxy(value) => JValue::String(value.to_string()),
+            OptionKind::Repeat(value) => value.to_json(),
             OptionKind::Resolve(value) => JValue::String(value.to_string()),
             OptionKind::Retry(value) => value.to_json(),
             OptionKind::RetryInterval(value) => value.to_json(),
@@ -315,8 +334,25 @@ impl ToJson for EntryOption {
             OptionKind::Verbose(value) => value.to_json(),
             OptionKind::VeryVerbose(value) => value.to_json(),
         };
-        attributes.push((name, value));
 
+        // If the value contains the unit such as `{ "value": 10, "unit": "second" }`
+        // The JSON for this option should still have one level
+        // for example: { "name": "delay", "value": 10, "unit", "second" }
+        let attributes = if let JValue::Object(mut attributes) = value {
+            attributes.push((
+                "name".to_string(),
+                JValue::String(self.kind.name().to_string()),
+            ));
+            attributes
+        } else {
+            vec![
+                (
+                    "name".to_string(),
+                    JValue::String(self.kind.name().to_string()),
+                ),
+                ("value".to_string(), value),
+            ]
+        };
         JValue::Object(attributes)
     }
 }
@@ -330,30 +366,42 @@ impl ToJson for BooleanOption {
     }
 }
 
-impl ToJson for NaturalOption {
+impl ToJson for CountOption {
     fn to_json(&self) -> JValue {
         match self {
-            NaturalOption::Literal(value) => JValue::Number(value.to_string()),
-            NaturalOption::Expression(expr) => expr.to_json(),
+            CountOption::Literal(value) => value.to_json(),
+            CountOption::Expression(expr) => expr.to_json(),
         }
     }
 }
 
-impl ToJson for RetryOption {
+impl ToJson for Count {
     fn to_json(&self) -> JValue {
         match self {
-            RetryOption::Literal(value) => value.to_json(),
-            RetryOption::Expression(expr) => expr.to_json(),
+            Count::Finite(n) => JValue::Number(n.to_string()),
+            Count::Infinite => JValue::Number("-1".to_string()),
         }
     }
 }
 
-impl ToJson for Retry {
+impl ToJson for DurationOption {
     fn to_json(&self) -> JValue {
         match self {
-            Retry::None => JValue::Number("0".to_string()),
-            Retry::Finite(value) => JValue::Number(value.to_string()),
-            Retry::Infinite => JValue::Number("-1".to_string()),
+            DurationOption::Literal(value) => value.to_json(),
+            DurationOption::Expression(expr) => expr.to_json(),
+        }
+    }
+}
+
+impl ToJson for Duration {
+    fn to_json(&self) -> JValue {
+        if let Some(unit) = self.unit {
+            let mut attributes =
+                vec![("value".to_string(), JValue::Number(self.value.to_string()))];
+            attributes.push(("unit".to_string(), JValue::String(unit.to_string())));
+            JValue::Object(attributes)
+        } else {
+            JValue::Number(self.value.to_string())
         }
     }
 }
@@ -718,6 +766,9 @@ impl ToJson for FilterValue {
                 attributes.push(("type".to_string(), JValue::String("toDate".to_string())));
                 attributes.push(("fmt".to_string(), JValue::String(fmt.to_string())));
             }
+            FilterValue::ToFloat => {
+                attributes.push(("type".to_string(), JValue::String("toFloat".to_string())));
+            }
             FilterValue::ToInt => {
                 attributes.push(("type".to_string(), JValue::String("toInt".to_string())));
             }
@@ -736,8 +787,16 @@ impl ToJson for Expr {
     }
 }
 
+impl ToJson for Comment {
+    fn to_json(&self) -> JValue {
+        JValue::String(self.value.to_string())
+    }
+}
+
 #[cfg(test)]
 pub mod tests {
+    use hurl_core::reader::Pos;
+
     use super::*;
 
     fn whitespace() -> Whitespace {
