@@ -16,6 +16,7 @@
  *
  */
 use hurl_core::ast::*;
+use hurl_core::typing::{Count, Duration};
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum Token {
@@ -41,6 +42,7 @@ pub enum Token {
     CodeDelimiter(String),
     CodeVariable(String),
     Lang(String),
+    Unit(String),
 }
 
 pub trait Tokenizable {
@@ -636,24 +638,45 @@ impl Tokenizable for PredicateValue {
 impl Tokenizable for MultilineString {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![Token::StringDelimiter("```".to_string())];
-        // FIXME: ugly if !let workaround, will be removed soon as
-        // OneLineText is temporary.
-        if let MultilineString::OneLineText(..) = self {
-        } else {
-            tokens.push(Token::Lang(self.lang().to_string()));
+        tokens.push(Token::Lang(self.lang().to_string()));
+        for (i, attribute) in self.attributes.iter().enumerate() {
+            if i > 0 || !self.lang().is_empty() {
+                tokens.push(Token::StringDelimiter(",".to_string()));
+            }
+            tokens.append(&mut attribute.tokenize());
         }
         match self {
-            MultilineString::OneLineText(template) => tokens.append(&mut template.tokenize()),
-            MultilineString::Text(text)
-            | MultilineString::Json(text)
-            | MultilineString::Xml(text) => tokens.append(&mut text.tokenize()),
-            MultilineString::GraphQl(graphql) => tokens.append(&mut graphql.tokenize()),
+            MultilineString {
+                kind: MultilineStringKind::Text(text),
+                ..
+            }
+            | MultilineString {
+                kind: MultilineStringKind::Json(text),
+                ..
+            }
+            | MultilineString {
+                kind: MultilineStringKind::Xml(text),
+                ..
+            } => tokens.append(&mut text.tokenize()),
+            MultilineString {
+                kind: MultilineStringKind::GraphQl(graphql),
+                ..
+            } => tokens.append(&mut graphql.tokenize()),
         }
         tokens.push(Token::StringDelimiter("```".to_string()));
         tokens
     }
 }
 
+impl Tokenizable for MultilineStringAttribute {
+    fn tokenize(&self) -> Vec<Token> {
+        match self {
+            MultilineStringAttribute::Escape => vec![Token::String("escape".to_string())],
+            MultilineStringAttribute::NoVariable => vec![Token::String("novariable".to_string())],
+        }
+    }
+}
+
 impl Tokenizable for Text {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![];
@@ -902,6 +925,7 @@ impl Tokenizable for OptionKind {
             OptionKind::Output(filename) => filename.tokenize(),
             OptionKind::PathAsIs(value) => value.tokenize(),
             OptionKind::Proxy(value) => value.tokenize(),
+            OptionKind::Repeat(value) => value.tokenize(),
             OptionKind::Resolve(value) => value.tokenize(),
             OptionKind::Retry(value) => value.tokenize(),
             OptionKind::RetryInterval(value) => value.tokenize(),
@@ -933,25 +957,43 @@ impl Tokenizable for NaturalOption {
     }
 }
 
-impl Tokenizable for RetryOption {
+impl Tokenizable for CountOption {
     fn tokenize(&self) -> Vec<Token> {
         match self {
-            RetryOption::Literal(retry) => retry.tokenize(),
-            RetryOption::Expression(expr) => expr.tokenize(),
+            CountOption::Literal(retry) => retry.tokenize(),
+            CountOption::Expression(expr) => expr.tokenize(),
         }
     }
 }
 
-impl Tokenizable for Retry {
+impl Tokenizable for Count {
     fn tokenize(&self) -> Vec<Token> {
         match self {
-            Retry::None => vec![Token::Number("0".to_string())],
-            Retry::Finite(n) => vec![Token::Number(n.to_string())],
-            Retry::Infinite => vec![Token::Number("-1".to_string())],
+            Count::Finite(n) => vec![Token::Number(n.to_string())],
+            Count::Infinite => vec![Token::Number("-1".to_string())],
         }
     }
 }
 
+impl Tokenizable for DurationOption {
+    fn tokenize(&self) -> Vec<Token> {
+        match self {
+            DurationOption::Literal(value) => value.tokenize(),
+            DurationOption::Expression(expr) => expr.tokenize(),
+        }
+    }
+}
+
+impl Tokenizable for Duration {
+    fn tokenize(&self) -> Vec<Token> {
+        let mut tokens = vec![Token::Number(self.value.to_string())];
+        if let Some(unit) = self.unit {
+            tokens.push(Token::Unit(unit.to_string()));
+        }
+        tokens
+    }
+}
+
 impl Tokenizable for VariableDefinition {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![Token::String(self.name.clone())];
@@ -1041,6 +1083,7 @@ impl Tokenizable for Filter {
                 tokens.append(&mut fmt.tokenize());
                 tokens
             }
+            FilterValue::ToFloat => vec![Token::FilterType(String::from("toFloat"))],
             FilterValue::ToInt => vec![Token::FilterType(String::from("toInt"))],
             FilterValue::XPath { space0, expr } => {
                 let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("xpath"))];
