@@ -15,12 +15,13 @@
  * limitations under the License.
  *
  */
+use crate::linter::{LinterError, LinterErrorKind};
 use hurl_core::ast::*;
-
-use crate::linter::core::{Error, LinterError};
+use hurl_core::reader::Pos;
+use hurl_core::typing::{Duration, DurationUnit};
 
 /// Returns lint errors for the `hurl_file`.
-pub fn check_hurl_file(hurl_file: &HurlFile) -> Vec<Error> {
+pub fn check_hurl_file(hurl_file: &HurlFile) -> Vec<LinterError> {
     hurl_file.entries.iter().flat_map(check_entry).collect()
 }
 
@@ -32,7 +33,7 @@ pub fn lint_hurl_file(hurl_file: &HurlFile) -> HurlFile {
     }
 }
 
-fn check_entry(entry: &Entry) -> Vec<Error> {
+fn check_entry(entry: &Entry) -> Vec<LinterError> {
     let mut errors = vec![];
     errors.append(&mut check_request(&entry.request));
     match &entry.response {
@@ -48,23 +49,24 @@ fn lint_entry(entry: &Entry) -> Entry {
     Entry { request, response }
 }
 
-fn check_request(request: &Request) -> Vec<Error> {
+fn check_request(request: &Request) -> Vec<LinterError> {
     let mut errors = vec![];
     if !request.space0.value.is_empty() {
-        errors.push(Error {
+        errors.push(LinterError {
             source_info: request.space0.source_info,
-            inner: LinterError::UnnecessarySpace,
+            kind: LinterErrorKind::UnnecessarySpace,
         });
     }
     if request.space1.value != " " {
-        errors.push(Error {
+        errors.push(LinterError {
             source_info: request.space1.source_info,
-            inner: LinterError::OneSpace,
+            kind: LinterErrorKind::OneSpace,
         });
     }
     for error in check_line_terminator(&request.line_terminator0) {
         errors.push(error);
     }
+    errors.extend(request.sections.iter().flat_map(check_section));
     errors
 }
 
@@ -96,14 +98,15 @@ fn lint_request(request: &Request) -> Request {
     }
 }
 
-fn check_response(response: &Response) -> Vec<Error> {
+fn check_response(response: &Response) -> Vec<LinterError> {
     let mut errors = vec![];
     if !response.space0.value.is_empty() {
-        errors.push(Error {
+        errors.push(LinterError {
             source_info: response.space0.source_info,
-            inner: LinterError::UnnecessarySpace,
+            kind: LinterErrorKind::UnnecessarySpace,
         });
     }
+    errors.extend(response.sections.iter().flat_map(check_section));
     errors
 }
 
@@ -133,6 +136,20 @@ fn lint_response(response: &Response) -> Response {
     }
 }
 
+fn check_section(section: &Section) -> Vec<LinterError> {
+    let mut errors = vec![];
+    if !section.space0.value.is_empty() {
+        errors.push(LinterError {
+            source_info: section.space0.source_info,
+            kind: LinterErrorKind::UnnecessarySpace,
+        });
+    }
+    for error in check_line_terminator(&section.line_terminator0) {
+        errors.push(error);
+    }
+    errors
+}
+
 fn lint_section(section: &Section) -> Section {
     let line_terminators = section.line_terminators.clone();
     let line_terminator0 = section.line_terminator0.clone();
@@ -278,7 +295,7 @@ fn lint_query_value(query_value: &QueryValue) -> QueryValue {
             attribute_name: field,
             ..
         } => QueryValue::Certificate {
-            attribute_name: field.clone(),
+            attribute_name: *field,
             space0: one_whitespace(),
         },
     }
@@ -422,14 +439,43 @@ fn lint_predicate_value(predicate_value: &PredicateValue) -> PredicateValue {
 
 fn lint_multiline_string(multiline_string: &MultilineString) -> MultilineString {
     match multiline_string {
-        MultilineString::OneLineText(value) => MultilineString::OneLineText(lint_template(value)),
-        MultilineString::Text(value) => MultilineString::Text(lint_text(value)),
-        MultilineString::Json(value) => MultilineString::Json(lint_text(value)),
-        MultilineString::Xml(value) => MultilineString::Xml(lint_text(value)),
-        MultilineString::GraphQl(value) => MultilineString::GraphQl(lint_graphql(value)),
+        MultilineString {
+            kind: MultilineStringKind::Text(value),
+            attributes,
+        } => MultilineString {
+            kind: MultilineStringKind::Text(lint_text(value)),
+            attributes: lint_multiline_string_attributes(attributes),
+        },
+        MultilineString {
+            kind: MultilineStringKind::Json(value),
+            attributes,
+        } => MultilineString {
+            kind: MultilineStringKind::Json(lint_text(value)),
+            attributes: lint_multiline_string_attributes(attributes),
+        },
+        MultilineString {
+            kind: MultilineStringKind::Xml(value),
+            attributes,
+        } => MultilineString {
+            kind: MultilineStringKind::Xml(lint_text(value)),
+            attributes: lint_multiline_string_attributes(attributes),
+        },
+        MultilineString {
+            kind: MultilineStringKind::GraphQl(value),
+            attributes,
+        } => MultilineString {
+            kind: MultilineStringKind::GraphQl(lint_graphql(value)),
+            attributes: lint_multiline_string_attributes(attributes),
+        },
     }
 }
 
+fn lint_multiline_string_attributes(
+    attributes: &[MultilineStringAttribute],
+) -> Vec<MultilineStringAttribute> {
+    attributes.to_vec()
+}
+
 fn lint_text(text: &Text) -> Text {
     let space = empty_whitespace();
     let newline = text.newline.clone();
@@ -567,15 +613,15 @@ fn one_whitespace() -> Whitespace {
     }
 }
 
-fn check_line_terminator(line_terminator: &LineTerminator) -> Vec<Error> {
+fn check_line_terminator(line_terminator: &LineTerminator) -> Vec<LinterError> {
     let mut errors = vec![];
     match &line_terminator.comment {
         Some(_) => {}
         None => {
             if !line_terminator.space0.value.is_empty() {
-                errors.push(Error {
+                errors.push(LinterError {
                     source_info: line_terminator.space0.source_info,
-                    inner: LinterError::UnnecessarySpace,
+                    kind: LinterErrorKind::UnnecessarySpace,
                 });
             }
         }
@@ -623,7 +669,44 @@ fn lint_template(template: &Template) -> Template {
 }
 
 fn lint_entry_option(entry_option: &EntryOption) -> EntryOption {
-    entry_option.clone()
+    EntryOption {
+        line_terminators: entry_option.line_terminators.clone(),
+        space0: empty_whitespace(),
+        space1: empty_whitespace(),
+        space2: one_whitespace(),
+        kind: lint_option_kind(&entry_option.kind),
+        line_terminator0: entry_option.line_terminator0.clone(),
+    }
+}
+
+fn lint_option_kind(option_kind: &OptionKind) -> OptionKind {
+    match option_kind {
+        OptionKind::Delay(duration) => {
+            OptionKind::Delay(lint_duration_option(duration, DurationUnit::MilliSecond))
+        }
+        OptionKind::RetryInterval(duration) => {
+            OptionKind::RetryInterval(lint_duration_option(duration, DurationUnit::MilliSecond))
+        }
+        _ => option_kind.clone(),
+    }
+}
+
+fn lint_duration_option(
+    duration_option: &DurationOption,
+    default_unit: DurationUnit,
+) -> DurationOption {
+    match duration_option {
+        DurationOption::Literal(duration) => {
+            DurationOption::Literal(lint_duration(duration, default_unit))
+        }
+        DurationOption::Expression(expr) => DurationOption::Expression(expr.clone()),
+    }
+}
+
+fn lint_duration(duration: &Duration, default_unit: DurationUnit) -> Duration {
+    let value = duration.value;
+    let unit = Some(duration.unit.unwrap_or(default_unit));
+    Duration { value, unit }
 }
 
 fn lint_filter(filter: &Filter) -> Filter {
